// -*-C++-*-
//
// File:      OmImageDisplay.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Sep  6 12:10:40 MEST 2002
// Usage:     -
// Info:      -

#include "OmImageDisplay.h"
#include <fstream>
#include <sstream>
//#include <unistd.h> // unlink()
#include <cstdio>
#include "AngleFormatter.h"
#include <qstring.h>
#include "OmSourceRegions.h"

extern F90MODVARDEF(OmSourceParameters, omsource_glue, mybackgroundsources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(OmSourceRegions, omsource_glue, mybackgroundregions);
char OmImageDisplay::_regionFile[100] = "\0";

/***************************************************************
*
****************************************************************/
void OmImageDisplay::toRegion(const OmSourceParameters& sources, 
                           const bool deleteRegions)
{
   
    if (deleteRegions) send("regions deleteall");
	
   

    ofstream os(_regionFile);

    os << "global color=yellow select=0 edit=0 move=0 delete=1" << endl;
   
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {


        OmSourceParameter& s = sources[id];   


        
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia  << ','
	   << s.semib << ','
	   << s.alpha << ')'  // Position angel of ellipse  
	   << " #text='" << id << "' color=cyan" 
	   << endl;

    }
   

    os.close();
    send("regions color cyan");
    send("regions file " + string(_regionFile));
   
   


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displayBackgroundRegions(const OmSourceParameters& sources)
{
           

    ofstream os(_regionFile);

    os << "global color=green select=0 edit=0 move=0 delete=1" << endl;
   
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {


        OmSourceParameter& s = sources[id];   


        
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia  << ','
	   << s.semib << ','
	   << s.alpha << ')'  // Position angel of ellipse  
	   << " #text='" << id << "' color=green" 
	   << endl;

    }
   

    os.close();
    send("regions color green");
    send("regions file " + string(_regionFile));
   
  


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displayBackgroundRegions(const OmSourceRegions*
     sources, const float radius)
{
       

    ofstream os(_regionFile);

    os << "global color=green select=0 edit=0 move=0 delete=1" << endl;
   
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {

           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
	   << radius << ')'  // Position angel of ellipse  
	   << " #text='" << i << "' color=green" 
	   << endl;

    }
   

    os.close();
    send("regions color green");
    send("regions file " + string(_regionFile));
   
    remove(_regionFile);


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displayBackgroundRegions(const OmSourceRegions*
     sources, const vector<float> &radius)
{
      

    ofstream os(_regionFile);

    os << "global color=green select=0 edit=0 move=0 delete=1" << endl;
   
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {

           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
 

	   << radius[i - 1] << ')'  // radius of circle  

	   << " #text='" << i << "' color=green" 
	   << endl;

    }
   

    os.close();
    send("regions color green");
    send("regions file " + string(_regionFile));
   


}

/***************************************************************
*
****************************************************************/
void OmImageDisplay::displaySourceRegions(const OmSourceParameters& sources)
{
  
    

    ofstream os(_regionFile);

    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
   
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {


        OmSourceParameter& s = sources[id];   


        
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia  << ','
	   << s.semib << ','
	   << s.alpha << ')'  // Position angel of ellipse  
	   << " #text='" << id << "' color=green" 
	   << endl;

    }
   

    os.close();
    send("regions color green");
    send("regions file " + string(_regionFile));
   


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displaySourceRegions(const OmSourceRegions*
     sources, const float radius)
{
   
 

    ofstream os(_regionFile);

    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
   
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {


           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
	   << radius << ')'  // Position angel of ellipse  
	   << " #text='" << i << "' color=green" 
	   << endl;

    }
   

    os.close();
    send("regions color green");
    send("regions file " + string(_regionFile));
   
    remove(_regionFile);


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displaySourceRegions(const OmSourceRegions*
     sources, const vector<float> &radius)
{
   

    ofstream os(_regionFile);

    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
   
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {


      //        OmSourceRegions & s = (*sources)[id];   

           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
 

	   << radius[i - 1] << ')'  // radius of circle  

	   << " #text='" << i << "' color=green" 
	   << endl;

    }
   

    os.close();
    send("regions color green");
    send("regions file " + string(_regionFile));
   


}

/***************************************************************
*
****************************************************************/
void OmImageDisplay::deleteRegions(const OmSourceRegions* sources)
{
   
  
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {


      //        OmSourceRegions & s = (*sources)[id];   

      char text[100];
      sprintf(text, "delete %f7.2 %f7.2", (*sources)[i].x(), (*sources)[i].y());
      send(text);	   

    }
   
}

/******************************************************
*
******************************************************/
void OmImageDisplay::zoomTo(const OmSourceParameter& s, int zoomFactor)
{
    std::stringstream ss;
   
    send("regions deleteall");
    ss << "pan to " << s.xcoord << ' ' << s.ycoord << " image";
    send(ss.str());
   
    
    char text[12];
    sprintf(text, "zoom to %d", zoomFactor);
    send(text);  // was 12
   
  


    ofstream os(_regionFile);
    float semiA = s.photometryradius * s.axialRatio;
    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
 
    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA  << ','
       << s.photometryradius << ','
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=blue" 
       << endl;

    semiA = s.innerbackgroundradius * s.axialRatio;
    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA    << ','  
       << s.innerbackgroundradius  << ',' 
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=cyan" 
       << endl;
  
    semiA = s.outerbackgroundradius * s.axialRatio;
    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA    << ','  
       << s.outerbackgroundradius  << ',' 
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=cyan" 
       << endl;
  
  

    os.close();
    // Send to ds9 
    send("regions file " + string(_regionFile));
     
}
/******************************************************
*
******************************************************/
void OmImageDisplay::displaySource(const int number, const OmSourceParameter& s)
{
    std::stringstream ss;

    

    send("regions deleteall");
    ss << "pan to " << s.xcoord << ' ' << s.ycoord << " image";
    send(ss.str());
    send("zoom to 15");
   

    ofstream os(_regionFile);


    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << s.semia  << ','
       << s.semib << ','
       << s.theta  

       << ')'
       << " # color=cyan"
       << endl;


  

  
    AngleFormatter af;
    af.setMode(AngleFormatter::HMS);
    QString text1(af.asString(s.rightascension).c_str());

    af.setMode(AngleFormatter::DMS);
    QString text2(af.asString(s.declination).c_str());
  

    QString text;
    text.sprintf("'Source %d RA=%s, DEC=%s'", number, (const char*)text1, (const char*)text2);
  
    os  << " text(" <<  (s.xcoord - 2.0)  << " "  << (s.ycoord + 12.0) << " " 
        <<  ((const char *)text) << " # color=black font='times 20 bold'" << endl;
      
    text.sprintf("'Magnitude= %5.2f+/-%5.2f'", s.magnitude, s.magnitudeerror);
    os  << " text(" <<  (s.xcoord - 4.0)  << " "  << (s.ycoord + 10.0) << " " 
        <<  ((const char *)text) << " # color=black font='times 20 bold'" << endl;
  

    os.close();
 
    send("regions file " + string(_regionFile));
          
}
/**********************************************************
*
***********************************************************/
void OmImageDisplay::plotPoint(const OmSourceParameter& s)
{


    std::stringstream ss;

    ss << "point(" << s.xcoord << " " << s.ycoord << ") # point=cross" << endl;
  
    send(ss.str());
  
 
   std::stringstream ss1;

    ss1 << " x point " << s.xcoord << ',' << s.ycoord;
    //  ss1 << " x point " << s.xcoord << ',' << s.ycoord;

    //    send(ss1.str());




}
/******************************************************************************
*
*
*****************************************************************************/
void OmImageDisplay::displayCurrentSourceList()
{
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
     

 
    toRegion(mySources);

}
