#include "OmSourceBackgroundDlg.h"
#include "OmSourcePhotometryDlg.h"

#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qtable.h>
#include <qstringlist.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qspinbox.h>
#include <qtooltip.h>
#include "OmSourceVersionNo.h"
#include "OmSourceParameters.h"
#include <iostream>
#include "OmSourceButton.h"
#include "OmSourceGroupBox.h"
#include "OmSourceLabel.h"
#include "OmSourceNumerics.h"
#include "OmImageDisplay.h"
#include "OmSourceRegions.h"
#include "average.h"
#include <cmath>

F90MODPROCPROTO(void,omsource_glue, deletebackgroundlist)();

F90MODPROCPROTO(void, omsource_glue, deletebackgroundregions)();

F90MODPROCPROTO(void, omsource_glue, addbackgroundregions)(OmSourceRegions *,const float &);
F90MODPROCPROTO(void, omsource_glue, getthecounts)();
F90MODPROCPROTO(void, omsource_glue, appendtobackgroundregions)(OmSourceRegions *);
F90MODPROCPROTO(float, omsource_glue, getimagecounts)(float &, float&, float&);


extern F90MODVARDEF(OmSourceParameters, omsource_glue, mybackgroundsources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(OmSourceRegions, omsource_glue, mybackgroundregions);

/********************************************************************************
 * Dialog for obtaining background regions from the user          
 *
 *******************************************************************************/
vector<float> BackgroundDlg::_radiusList;
BackgroundDlg::BackgroundDlg(QWidget *parent,  OmImageDisplay *imageDisplay,
    OmSourceParameter *source, int zoomFactor) :
    QDialog(parent, "", true), _imageDisplay(imageDisplay),
    _source(source), _zoomFactor(zoomFactor), 
    _apertureRadius(6.0)

{

  //    setCaption("PLEASE SELECT BACKROUND REGIONS BY CLICKING LEFT-MOUSE BUTTON OUTSIDE THE PHOTOMETRY ANNULI"); 
    // Create the table for storing details about the selected background regions 


    QString text("Please select background regions by\n"
                 "clicking the left mouse-button\n"
                 "The mean counts/pixel of the selected\n"
                 "regions (shown below) will be used to\n"
                 "compute the background counts of the source");
    OmSourceLabel *label = new OmSourceLabel(this, text, 20, 30, 270, 100);



    int xPos = 20;
    int yPos = label->geometry().bottom() + 20;
    int height = 30;

    _averageTxt = new OmSourceLabel(this, "Mean counts/pixel=0+/-0 (n=0)",  xPos, yPos, height);
  
  



    _table = new QTable(10, 3, this, 0);
    yPos = _averageTxt->geometry().bottom() + 20;
  
    _table->setGeometry(20, yPos, 350, 120);
    yPos = _table->geometry().bottom() + 20;
  

    int noColumns = 5;


    _table->setReadOnly(true);   // Can only read the table
    _table->setNumCols( noColumns);
    _table->setNumRows(1);   
   

    QHeader *th = _table->horizontalHeader();
   
 
    
    th->setLabel(0,"x coord");
    _table->setText(0, 0, "x coord");
 

    th->setLabel(1,"y coord");
    _table->setText(0, 1, "y coord");
 
    th->setLabel(2,"radius");
    _table->setText(0, 2, "radius");
 

    th->setLabel(3,"counts");
    _table->setText(0, 3, "counts");


    th->setLabel(4,"+/-");
    _table->setText(0, 4, "counts");



    /********************************************
    * Adjust the size of the cells
    *******************************************/
    for (int i = 0; i < noColumns; i++)
    {         
        _table->setColumnWidth(i, _table->text(0, i).length() * 10);
	_table->setText(0, i, " ");
    }
    _table->setNumRows(0);   
 
  
    OmSourceGroupBox *group = new OmSourceGroupBox( this, 
          "List operations", 20, yPos, 
           120, 60, "An operation that is performed on the background-list" );   
						  

    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
   
 
    _deleteListBtn = new OmSourceButton(group, "Delete",
               "Click here to delete the source list");
  
    layout->addWidget( _deleteListBtn);
  
    connect(_deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );


     /***********************************************************
     * Spin box to get the radius of the background aperture
     **********************************************************/
    xPos = 20;
    yPos = group->geometry().bottom() + 30;
  
    OmSourceGroupBox *groupBox = new OmSourceGroupBox( this,  "Annulus (pixels)", xPos, yPos, 160, 70);
   
    xPos = 10;
    yPos = 20;
    _apertureTxt = new OmSourceLabel(groupBox, "Aperture",  xPos, yPos, height);
 
 
    xPos += 80; 
  
    _apertureSpinBox = new QSpinBox(groupBox, "SpinBox2" );
    _apertureSpinBox->setGeometry( xPos, yPos, 50, height ); 
    
    _apertureSpinBox->setMaxValue(30);
    _apertureSpinBox->setMinValue(3);
    _apertureSpinBox->setValue((int)_apertureRadius);

    connect(_apertureSpinBox, SIGNAL(valueChanged(int)), this, SLOT(apertureRadiusHasChanged(int)));
   
    text.sprintf("Change the background aperture radius\n (allowed values=%d-%d)", 
		 3, 30);
    QToolTip::add(_apertureSpinBox, text);

    yPos = groupBox->geometry().bottom() + 30;

    OmSourceButton* okButton = new OmSourceButton(this, "OK", 
         "Click here when you have selected your background regions\n"
         "The average of the mean counts/pixels for each region\n"
	 "will be used in the background-subtraction");
    okButton->setGeometry(100, yPos, 100, 30);
    connect(okButton, SIGNAL( clicked() ), this, SLOT( accept() ));
 
   _average = new Average;
 
  

 
    _imageDisplay->zoomTo(*_source, _zoomFactor);

   

    update();

    OmSourceRegions *points1 = OmImageRegions::GetNewRegions(_imageDisplay);
    if (points1) delete points1;	  

  
    _timer = new QTimer( this);

    connect( _timer, SIGNAL(timeout()), this, SLOT(updateRegions()) );
  
    _timer->start( 3000); // 2 seconds single-shot timer
      
      
};
/***************************************
 * Destructor
 **************************************/
BackgroundDlg::~BackgroundDlg()
{

    _timer->stop();
    delete _timer;
    delete _average;
}

/**********************************************
 *
 *********************************************/
void BackgroundDlg::deleteList()
{
   _timer->stop();

   cout << "Backgrounddlg deleteList line 201\n";
   _imageDisplay->send("regions deleteall");

   // Reset the table  
   _table->setNumRows(0);   
  
    _deleteListBtn->setDisabled(TRUE);
    // Delete the background regions

  
    F90MODPROC(omsource_glue, deletebackgroundlist)();
    F90MODPROC(omsource_glue, deletebackgroundregions)();
    delete OmImageRegions::_backgroundRegions;
  
 
    OmImageRegions::_noBackgroundRegions = 0;

    _radiusList.clear();
   
    _averageTxt -> setText(" ");



    _average->reset();
    _imageDisplay->zoomTo(*_source, _zoomFactor);
    OmSourceRegions *points1 = OmImageRegions::GetNewRegions(_imageDisplay);
    if (points1) delete points1;	  
 

    _timer->start( 5000);
 
     
}
/************************************************************
*
*
***********************************************************/
void BackgroundDlg::mousePressEvent( QMouseEvent *e )
{
    QApplication::beep();
    if ( e->button() == QMouseEvent::LeftButton )               // left button pressed
    QMessageBox::critical( 0, "omsource",
			       QString("LEFT MOUSE BUTTON")); 

}
/*********************************************************
* Checks the image for any new background regions
* and, if necessary, updates the image and table
********************************************************/
void BackgroundDlg::updateRegions()
{ 
  cout << "Backgrounddlg updateRegions line 252\n";
    
  //  QApplication::beep();
 
    // Get the new background regions
    OmSourceRegions *newRegions = OmImageRegions::GetNewRegions(_imageDisplay);

 
    if (newRegions)
      { 
       
 QApplication::beep();
  cout << "Backgrounddlg updateRegions line 264\n";
 
        OmImageRegions::updateBackgroundRegions(newRegions);
      
      }	
    else
	return;
   

    // Display the selected source
    _imageDisplay->zoomTo(*_source, _zoomFactor);
    
 


    // Compute the counts for each newly selected region
    float standardDeviation;
     
    for (int i = newRegions->lbound(); i <= newRegions->ubound(); i++)
    {   
  

        float x = (*newRegions)[i].x();
        float y = (*newRegions)[i].y();
           
        float counts = F90MODPROC(omsource_glue, getimagecounts)(x, y, _apertureRadius);
	  
        float countsError = sqrt(counts);
        // Update the table
        updateTable(x, y, _apertureRadius, counts, countsError);  
        _radiusList.push_back(_apertureRadius);
        counts /= M_PI * _apertureRadius * _apertureRadius;
        countsError = sqrt(counts);
       	
	_backgroundMode = _average->UpdateAverage(counts, countsError, standardDeviation);
    }
    delete newRegions;  // Finished with the new background regions
 
  

    _imageDisplay->displayBackgroundRegions(OmImageRegions::_backgroundRegions, _radiusList);

      
    QString text;
    text.sprintf("Mean counts/pixel=%7.2f+/-%7.2f (n=%d)", 
            _backgroundMode, standardDeviation, OmImageRegions::_noBackgroundRegions); 
   
    _averageTxt -> setText(text);

     
}

/*****************************************************
 * Called when the value of apertureradius 
 *spin-box has changes
 ****************************************************/
void BackgroundDlg::apertureRadiusHasChanged(int radius)
{
    _apertureRadius = radius;
    _apertureSpinBox->setValue(radius);

}
/***********************************************************
*
*
***********************************************************/
void BackgroundDlg::updateTable(float x, float y, float radius, 
           float counts, float countsError)
{
    int rowNo = _table->numRows();
    
    _table->setNumRows(rowNo + 1);
    
    QString text;
                 
    text.sprintf("%6.2f", x);
    _table->setText(rowNo, 0, text);

    text.sprintf("%6.2f", y);
    _table->setText(rowNo, 1, text);

    text.sprintf("%2d",  (int)radius);
   _table->setText(rowNo, 2, text);

    text.sprintf("%8.2f", counts);  
   _table->setText(rowNo, 3, text);

    text.sprintf("%8.2f", countsError);  
   _table->setText(rowNo, 4, text);


}

/********************************************************
*
********************************************************/
void BackgroundDlg::update()
{
   
  
   if (OmImageRegions::_noBackgroundRegions == 0) return;
  

    _imageDisplay->displayBackgroundRegions(OmImageRegions::_backgroundRegions, _radiusList);
  

    // Compute the counts for each newly selected region

   _average->reset();

    float standardDeviation;
     
    for (int i = OmImageRegions::_backgroundRegions->lbound(); i <= OmImageRegions::_backgroundRegions->ubound(); i++)
    {   
 

        float x = (*OmImageRegions::_backgroundRegions)[i].x();
        float y = (*OmImageRegions::_backgroundRegions)[i].y();
           
        float counts = F90MODPROC(omsource_glue, getimagecounts)(x, y, _radiusList[i - 1]);
        float countsError = sqrt(counts);

          // Update the table
        updateTable(x, y, _radiusList[i - 1], counts, countsError);  
	

        counts /=  M_PI * _radiusList[i - 1] * _radiusList[i - 1];  
        countsError = sqrt(counts);


	_backgroundMode = _average->UpdateAverage(counts, countsError, standardDeviation);
    }

  
   
    QString text;
    text.sprintf("Mean counts/pixel=%7.2f+/-%7.2f (n=%d)", 
            _backgroundMode, standardDeviation, OmImageRegions::_noBackgroundRegions); 
   
    _averageTxt -> setText(text);


} 








