// -*-C++-*-
//
// File:      OmSourceBrowser.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Sep  6 11:14:45 MEST 2002
// Usage:     -
// Info:      -
// Omsource header files
#include <qcursor.h>
#include "OmSourceParameters.h"
#include "OmSourceVersionNo.h"
#include "OmSourceBrowser.h"

#include "OmSourceRegions.h"
#include "OmImageDisplay.h"
#include "OmProgressDialog.h"
#include "OmSourceLog.h"
#include "OmSourceNumerics.h"

#include "OmSourceDetectDialog.h"
#include "OmSourcePhotometryDlg.h"
//#include "OmSourceDeconvolveDlg.h"
#include "OmSourceInformationDlg.h"
#include "OmSourceProcessDlg.h"
//#include "OmSourceHistoryDlg.h"
#include "OmSourceTableDlg.h"
#include "OmSourceAddSourcesDlg.h"
#include "OmSourceLabel.h"
// OM header files
#include <omdetect.h>
#include <ommag.h>
#include <omatt.h>
// Other SAS header files
#include <Dal.h>

#include <ExceptionErrorHandler.h>
#include <param.h>
#include <errstr.h>
#include "AngleFormatter.h"

// QT header files
#include <qapplication.h>

#include <qlistbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qspinbox.h> 
#include <qstatusbar.h>
#include <qprogressbar.h>
#include <qstring.h>
#include <qtooltip.h>
#include <qinputdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qfiledialog.h>
#include <qprinter.h> 
#include <qpainter.h>
#include <qfont.h> 
#include <qpaintdevicemetrics.h> 
#include <qdatetime.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qprocess.h>
#include <qsettings.h>
#include <qstringlist.h>
#include <qfileinfo.h>
#include <qwhatsthis.h>
#include "allocate.h"
#include <iostream>
#include "OmSourceUtility.h"
#include "OmSourceButton.h"
#include "OmSourceGroupBox.h"
#include "OmSourceInformationMenu.h"
#include "OmSourceHelp.h"
#include "OmSourceRegions.h"
#include "OmSourceTempFile.h"

// --- Glue prototypes ---
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(PhotometryParameters, omsource_glue, myphotometryparameters);
extern F90MODVARDEF(float, omsource_glue, mycalradius);

F90MODPROCPROTO(void, omsource_glue, init)(bool &loaded);


F90MODPROCPROTO(void, omsource_glue, replace)(const int& id, const OmSourceParameter&);
F90MODPROCPROTO(void, omsource_glue, finish)();
F90MODPROCPROTO(void, omsource_glue, computebackground)(float &value, float &error);
F90MODPROCPROTO(void, omsource_glue, computejuliandates)(double &JD1, double &JD2, double &JD2);

F90MODPROCPROTO(void,omsource_glue, loadsources)(const char []);
F90MODPROCPROTO(void,omsource_glue, deletelist)();
F90MODPROCPROTO(void,omsource_glue, recompute)(const int& id);
F90MODPROCPROTO(void,omsource_glue, remove)(const int& id);
F90MODPROCPROTO(void,omsource_glue, addregions)(OmSourceRegions*);
F90MODPROCPROTO(void, omsource_glue, loadimage)(float**);
 

float** LoadImage(char *fileName, int &nx, int &ny);

F90MODPROCPROTO(void, omsource_glue, test1)(int size, float *data);

/*******************************************************
* constructor
* This is the main widget of the application*
******************************************************/
OmSourceBrowser::OmSourceBrowser(QWidget* parent, const char* name) : 
  QWidget(parent, name), _sourceNo(0), _sourceListHasChanged(false),  
  _sourceListIsLoaded(false)
{ 
 


 

    // Get the name of the image file name
    readFileNames();


    string file = stringParameter("imageset");
   
  
    strcpy(_imageFileName, file.c_str()); 
    if (file.length() < 2) 
    {
   
        displayImageFileMenu();
    }

    string text = "Initialising- please wait";

    qApp->processEvents();
 
    
    _reporter = new OmProgressDialog(text, 18);
    
    _reporter->setProgress(1);
   
     // load display
    _display = new OmImageDisplay;
     
    _reporter->setProgress(2);
  
 
    if (!_display->isUp())
        errstr << "Could not launch Imageviewer(ds9)" << Sas::error("NoDisplayer");
    

            _display->displayImage(_imageFileName);
	    



   
    _display->send("zoom to fit");
    

    _display->send("cmap heat"); 
    _display->send("scale zscale");
    _display->send("scale linear");
        
  





    _reporter->setProgress(3);
      
    
    _reporter->setProgress(4);
      
    _changed = FALSE;
   
    
    _reporter->setProgress(5);




    {
      


    OmSourceGroupBox *group = new OmSourceGroupBox( this, 
          "Source list operations", 20, 20, 200, 60, "An operation that is performed on the source-list" );   
						  

    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
   
    _sourceTableBtn = new OmSourceButton(group, "Table",
                "Click here to display the source-list table" );
    
    layout->addWidget( _sourceTableBtn);
    connect(_sourceTableBtn, SIGNAL(clicked()), this, SLOT (showSourceTable()) );
    _sourceTableBtn->setDisabled(TRUE);
   
  

    _reporter->setProgress(11);
 	
  
   
    _deleteListBtn = new OmSourceButton(group, "Delete",
               "Click here to delete the source list");
  
    layout->addWidget( _deleteListBtn);
  
    connect(_deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );
 

    _deleteListBtn->setDisabled(TRUE);
    _reporter->setProgress(12);



    }
    

    // create the image buttons
    {
    OmSourceGroupBox* group = new OmSourceGroupBox( this, "Image operations", 
                    20, 100, 320, 60, "Operations involving the whole image" );
 
  
  
    _reporter->setProgress(5);


     QHBoxLayout* layout = new QHBoxLayout(group, QHBoxLayout::LeftToRight);
    
     layout->setMargin(10);
     layout->setSpacing(10);  
    
   
    _viewAllBtn= new OmSourceButton(group, "View all", "Click here to view the whole image");

    
    layout->addWidget(  _viewAllBtn, 1);
    
    _viewAllBtn ->setDisabled(TRUE);
    connect(_viewAllBtn, SIGNAL(clicked()), this, SLOT  (viewAll()));
 

    _reporter->setProgress(6);
 
    _addSourcesBtn = new OmSourceButton(group, "Select sources", 
             "Click here to select additional sources\n"
             "from the image to add to the source list");
 
    layout->addWidget(  _addSourcesBtn, 2);
  
  
    connect(_addSourcesBtn, SIGNAL(clicked()), this, SLOT  (addSources()));
    _reporter->setProgress(7);
 
    _detectSourcesBtn = new OmSourceButton(group, "Run omdetect",
               "Click here to run the task omdetect on\n"
               "the image to populate the source list" );
  
    layout->addWidget(  _detectSourcesBtn, 3);
  
 
    connect(_detectSourcesBtn, SIGNAL(clicked()),
    	  this,   SLOT  (detectSources()));
     

    }
   
    // Group box for individual source operations
    {
    OmSourceGroupBox* group = new OmSourceGroupBox( this, "Individual source operations", 
           20, 180, 320, 60, "An operation that is performed on the selected source from the list");


    QHBoxLayout* layout = new QHBoxLayout(group, QHBoxLayout::LeftToRight);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
    

    // The photometry button
    _photometryBtn = new OmSourceButton(group, "Photometry",  
            "Click here to do interactive photometry on the selected source");
    layout->addWidget(_photometryBtn, 1);
 

    _photometryBtn->setDisabled(TRUE);
   
    connect(_photometryBtn, SIGNAL(clicked()), this, SLOT (photometry()));
         

    // The source deletion button
    _deleteSourceBtn = new OmSourceButton(group, "Delete", 
         "Click here to delete the selected source");
    layout->addWidget(_deleteSourceBtn, 2);
 
    _deleteSourceBtn->setDisabled(TRUE);
  
    connect(_deleteSourceBtn, SIGNAL(clicked()), this, SLOT (deleteSource()));

    // The source deletion button
    _viewSourceBtn = new OmSourceButton(group, "View", 
                "Click here to zoom in on the source");
  
    layout->addWidget(_viewSourceBtn, 3);
 
  
    _viewSourceBtn->setDisabled(TRUE);
  
    connect(_viewSourceBtn, SIGNAL(clicked()), this, SLOT (displaySource()));

    _reporter->setProgress(8);
  
    // Load the sources (if old sourcelist file specified)
    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);
 
    _reporter->setProgress(9);
 
  
    _reporter->setProgress(10);
  
  
    }

    /**********************************************************
     * Group box for input files info
     *********************************************************/
    int xPos, yPos;
    {
    OmSourceGroupBox* group = new OmSourceGroupBox( this, "Input files", 
               20, 260, 250, 60, " " );

    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  

    _loadImageBtn= new OmSourceButton(group, "Load image", 
             "Click here to select a new image file");    
  
    //    _loadImageBtn->setGeometry(10, 130, 70, 20);
    connect(_loadImageBtn, SIGNAL(clicked()), this, SLOT (loadImage()) );
   
    layout->addWidget( _loadImageBtn);
  
  
    _loadSourceListBtn= new OmSourceButton(group, "Load source-list", 
             "Click here to select a new source-list file");    
    
    connect(_loadSourceListBtn, SIGNAL(clicked()), this, SLOT (loadSourceList()) );
    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
    
    layout->addWidget(_loadSourceListBtn );
  
    // Information button
    
    yPos = group->geometry().bottom() + 20;
    }
    xPos = 20;
    OmSourceButton *informationButton = new OmSourceButton(this, "Information", xPos, yPos, 25,  
                 "Click here to display observation/image information");    
    connect(informationButton, SIGNAL(clicked()), this, SLOT (displayInformation()) );
  

    OmSourceLabel *label;
    
    yPos = informationButton->geometry().bottom() + 20;
    _group = new OmSourceGroupBox( this, "Current source", 20, yPos, 290, 280,
        "You select the source from the list for further operations" );   
    
  
    _group->setDisabled(TRUE);
    _reporter->setProgress(13);
   
    xPos = 10;
    yPos = 30;    
    {
    label = new OmSourceLabel( _group, "X", xPos, yPos, 25, "The X-coord of the selected source" );
    xPos = label->geometry().right() + 10;
   
 
    }
    _xCoord = new QLineEdit(_group, "xcoord");
    _xCoord->setReadOnly( true );
    //     yPos += 25;
    _xCoord->setGeometry(xPos, yPos, 60, 25); 
    _xCoord->setDisabled(true);
    _xCoord->setAlignment(Qt::AlignVCenter);
 
   //    yPos += 25; 
    yPos = _xCoord->geometry().bottom() + 10;
   

    {
    xPos = 10;

   
    label = new OmSourceLabel( _group, "RA", xPos, yPos, 25,  "The Right-Ascension of the selected source" );
    xPos = label->geometry().right() + 10;
 
    }   
   
    _ascension = new QLineEdit(_group, "ascension");
    _ascension->setReadOnly( true );
    //    yPos += 25;
  
    _ascension->setGeometry(xPos, yPos, 90, 25); 
    _ascension->setDisabled(true);
    _ascension->setAlignment(Qt::AlignVCenter);
 
    
    {
    
    xPos = 130;  
    xPos = _ascension->geometry().right() + 20;
  
    label = new OmSourceLabel( _group, "dec", xPos, yPos, 25,  "The declination of the selected source" );

 
    }
    
    //    yPos += 25;

    _declination = new QLineEdit(_group, "declination");
    _declination->setReadOnly( true );
  
    _declination->setGeometry(label->geometry().right() + 10, yPos, 90, 25); 
    _declination->setDisabled(true);
    _declination->setAlignment(Qt::AlignVCenter);
 


    {
  
      //    xPos = 130;
    
      yPos = _xCoord->geometry().top();
  
     label = new OmSourceLabel( _group, "Y", xPos, yPos, 25,  "The Y-coord of the selected source" );
     xPos = label->geometry().right() + 10;
 
    } 
  


    _yCoord = new QLineEdit(_group, "ycoord");
    _yCoord->setReadOnly( true );

    _yCoord->setGeometry(xPos, yPos, 60, 25 ); 
    _yCoord->setDisabled(true);
    _yCoord->setAlignment(Qt::AlignVCenter);
 
   
 
    



     xPos = 10;
  
     yPos = _declination->geometry().bottom() + 10;
 
    {
    label = new OmSourceLabel( _group, "Instrumental mag", xPos, yPos, 25,  
             "The instrumental magnitude of the selected source" );
    xPos = label->geometry().right() + 10;
 
    }   
    _magnitude = new QLineEdit(_group, "magnitude");
    _magnitude->setReadOnly( true );
    //    yPos += 25;
  
    _magnitude->setGeometry(xPos, yPos, 90, 25); 
    _magnitude->setDisabled(true);
    _magnitude->setAlignment(Qt::AlignVCenter);
    yPos = _magnitude->geometry().bottom() + 20;
 
    
    {
    
    xPos = 50;
    
    label = new OmSourceLabel( _group, "Source no", xPos, yPos, 25, "The selected source for editing\n"
                          "The number corresponds to the number on the image");

 
    }
    _reporter->setProgress(14);
  
    /******************************************************************
    * A spin box to set the source number
    *****************************************************************/
    xPos += label->width() + 10;
    _spinBoxNumber = new QSpinBox(_group, "a");
    _spinBoxNumber->setGeometry(xPos, yPos, 60, 25);
 

    connect(_spinBoxNumber, SIGNAL(valueChanged(int)), this, SLOT (selectSource(int)));

    connect(_spinBoxNumber, SIGNAL(valueChanged(int)), this, SLOT(  setListBoxItem(int)));

    
    QToolTip::add( _spinBoxNumber, "Change the source number to edit");
    _spinBoxNumber->show();
    _spinBoxNumber->setDisabled(TRUE);

    _group->resize(_group->width(), _spinBoxNumber->geometry().bottom() + 20);
    _reporter->setProgress(15);
 
    // Exit button
  
    xPos = max(20, rect().width() / 3 - 50);
    yPos = _group->geometry().bottom() + 20;
   
    _exitBtn= new OmSourceButton(this, "Exit", xPos, yPos, 70, 25,  
                 "Click here to exit the program and save any changes");    
    connect(_exitBtn, SIGNAL(clicked()), this, SLOT (okay()) );
  
 
    xPos += 90;
    _cancelBtn= new OmSourceButton(this, "Cancel", xPos, yPos, 70, 25,  
                 "Click here to exit the program without saving any changes");    
  
  
    connect(_cancelBtn, SIGNAL(clicked()), parent, SLOT (Exit()) );
    xPos += 90;
    _restartBtn= new OmSourceButton(this, "Restart", xPos, yPos, 70, 25, 
             "Click here to restart");    
  
    
    connect(_restartBtn, SIGNAL(clicked()), this, SLOT (restart()) );
   
    
    _reporter->setProgress(16);
  
    
    Initialise();
   
    setFocusPolicy( QWidget::ClickFocus );
  
       
}
/*******************************************************
* 
*******************************************************/
void OmSourceBrowser::Initialise()
{ 
 

    refreshListBox();
      
 
    _reporter->setProgress(17);
  
 
    DataSet *set = dataSetServer->open(string(_imageFileName), Dal::Read);

    if (set->hasAttribute("TELESCOP"))
        _obsDetails.telescope = (set->attribute("TELESCOP")->asString()).c_str();
 
    if (set->hasAttribute("INSTRUME"))  
        _obsDetails.instrument = (set->attribute("INSTRUME")->asString()).c_str();


    if (set->hasAttribute("OBS_ID"))
        _obsDetails.obsId = (set->attribute("OBS_ID")->asString()).c_str();

    if (set->hasAttribute("EXP_ID"))  
        _obsDetails.expId = (set->attribute("EXP_ID")->asString()).c_str();
 
    if (set->hasAttribute("DATE-OBS"))
       _obsDetails.dateObs = (set->attribute("DATE-OBS")->asString()).c_str();

    if (set->hasAttribute("DATE-END"))
        _obsDetails.dateEnd = (set->attribute("DATE-END")->asString()).c_str();

    if (set->hasAttribute("FILTER"))
        _obsDetails.filter = (set->attribute("FILTER")->asString()).c_str();
    
    if (set->hasAttribute("OBJECT"))
        _obsDetails.object = (set->attribute("OBJECT")->asString()).c_str();
 

    if (set->hasAttribute("OBSERVER"))
        _obsDetails.observer = (set->attribute("OBSERVER")->asString()).c_str();

    if (set->hasAttribute("EXPOSURE"))
        _obsDetails.exposure = (float)set->attribute("EXPOSURE")->asInt();

    if (set->hasAttribute("BINBPE"))
        _imageDetails.binBpe = set->attribute("BINBPE")->asBool();

    if (set->hasAttribute("BINAX1"))
        _imageDetails.binAx1 = set->attribute("BINAX1")->asInt();

    if (set->hasAttribute("BINAX2"))
        _imageDetails.binAx2 = set->attribute("BINAX2")->asInt();
    
    if (set->hasAttribute("FRAMTIME"))
          _imageDetails.frameTime = set->attribute("FRAMTIME")->asReal();

    if (set->hasAttribute("DEADFRAC"))
          _imageDetails.deadFraction = set->attribute("DEADFRACTION")->asReal();
   
  
    
    dataSetServer->close(set);
           
    // Compute the global background
    float error;
    F90MODPROC(omsource_glue, computebackground)(_imageDetails.background, error);
    error = 0.0;



    /*********************************************************
     * Compute the start/end/mid-point JDs
     *********************************************************/
  
     F90MODPROC(omsource_glue, computejuliandates)(_obsDetails.JDStart, 
		 _obsDetails.JDEnd, _obsDetails.JDMiddle);
 



    _reporter->setProgress(18);
 
    delete _reporter;
    _reporter = 0;

   
 
    _loadImageBtn->setEnabled(true);
    //   OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
   
  

    //        float **image = 0;
    //        F90MODPROC(omsource_glue, loadimage)(image);
  
    //        if (image) cout << image[0][0] << endl;
  
}
/******************************************************
* destructor
*******************************************************/

OmSourceBrowser::~OmSourceBrowser()
{
    _display->down();
    delete _display;

    //    close();
    saveFileNames();      
}
/****************************************************
 *
 *
 ***************************************************/
void OmSourceBrowser::close()
{
    // Save the name of the image file in the settings
  // for next run
    addFileToList(_imageFileList, _imageFileName);

    //    cout << "OmSourceBrowser close line 616 _newSourceListFile= " <<  _newSourceListFile << endl;


    if (_sourceListHasChanged)
    {
      if (getNewSrcListFile())
	{
	 

          QString text= "Creating new source-list file (" + QString((const char *)_newSourceListFile) + 
                    ")\n omatt will run\n\n - please wait"; 
          _newSourceFileList += _newSourceListFile;

	  ProcessDlg* finishProcessDlg = new ProcessDlg(this, 0, text, 
               ProcessDlg::createNewSourceListFile);    

            int retValue = finishProcessDlg->exec();
    
     
           delete finishProcessDlg;  
           switch (retValue)
           {
               case (FAILED):
    
	          return;
      
              default:
                  return;


           }
	}
    }
    else
    {
      
      F90MODPROC(omsource_glue, deletelist)();
 
    }
}
/********************************************************
 * selectSource
 *******************************************************/
void OmSourceBrowser::selectSource(int no) 
{
    

    _sourceNo = no;
    _photometryBtn->setEnabled(TRUE);
    _deleteSourceBtn->setEnabled(TRUE);
    //    _deconSourceBtn->setEnabled(TRUE);
       

    setSourceCoordinates();
    //    updateDisplay();
   


}; 
/********************************************************
 * setListBoxItem
 *******************************************************/
void OmSourceBrowser::setListBoxItem(int index) 
{
    _sourceNo = index;
    //    _list->setCurrentItem(index - 1); 
    _photometryBtn->setEnabled(TRUE);
    _deleteSourceBtn->setEnabled(TRUE);
    //    _deconSourceBtn->setEnabled(TRUE);
 
};
/*****************************************************
* SetSpinBoxValue
******************************************************/
void OmSourceBrowser::SetSpinBoxValue(int num)   
{ 
    _spinBoxNumber->setValue(num + 1); 

}

/*************************************************************
 * deleteList
 * Purpose - deletes all the sources from the list
 ************************************************************/
void OmSourceBrowser::deleteList()
{
  

    F90MODPROC(omsource_glue, deletelist)();
 

   _loadSourceListBtn->setEnabled(true);
 
  
    refreshListBox();


    viewAll();
}
/******************************************************
* recompute
* This is activated from OmSourceEditor when the apply
* button is clicked.
*******************************************************/
void OmSourceBrowser::recompute(const OmSourceParameter& s)
{
 
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
    mySources[_sourceNo] = s;
 

    F90MODPROC(omsource_glue, recompute)(_sourceNo);
    
    /*********************************************************************
    * Replace the source in the list box with the computed source values
    *********************************************************************/

    refreshListBox();
    _display->zoomTo(s);
 

    _changed = TRUE;

    // ************************************************NB 
    //    edit(_current - 1);
    SetSpinBoxValue(_sourceNo - 1);    
}
/******************************************************
* deleteSource
*******************************************************/
void OmSourceBrowser::deleteSource()
{
  // update the scrollview AND mySources!
     _sourceListHasChanged = true;
  
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    if (!_sourceNo || mySources.length() < _sourceNo) return;

   

    F90MODVAR(omsource_glue, remove)(_sourceNo);
    
 
    refreshListBox();
    viewAll();    
  

}
/***********************************************************
* viewAll
************************************************************/
void OmSourceBrowser::viewAll()
{
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
        _display->send("zoom to fit");


   _display->send("cmap heat"); 
   _display->send("scale zscale");
   _display->send("scale linear");
        





    //       _display->send("zoom to fit frame");
      
  


   


    _display->toRegion(mySources);

}
/********************************************************
* addSources
* Displays teh dialogue to allow the user
* to add sources to the list
*******************************************************/
void OmSourceBrowser::addSources()
{

    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
 
 
 //    emit SetCaption(string);
    if (mySources.ubound() > 0)
        SetSpinBoxValue(mySources.ubound());
 //        selectSource(mySources.ubound());
  
  
     viewAll();
     /*****************************************************************************
     * Get the photometry aperture radius
     ***************************************************************************/
    PhotometryParameters photParameters = F90MODVAR(omsource_glue, myphotometryparameters);
    
  
    AddSourcesDlg *addSourcesDlg = new AddSourcesDlg(this, _display, photParameters.apertureRadius);

    addSourcesDlg->move(0, 0);
    

    int retValue = addSourcesDlg->exec();
   
     
    delete addSourcesDlg;  
    switch (retValue)
    {
        case (QDialog::Accepted):
          break;  
	  //	  return;
    case (QDialog::Rejected):
      for (int i = 0; i < 10; i++)
	QApplication::beep();
    
        refreshListBox();

	    return;  
        default:
          break;
    };

    _sourceListHasChanged = true;
    
 
    // Get the user selected points from the image
    OmSourceRegions *points = OmImageRegions::GetRegions(_display); 
    
 
 
    if (!points->ubound())
    {
     
        QString text= QString("Please select sources manually (click Add sources button)") +
                      QString(" or click Run omdetect button to detect sources");
    
 
	    emit SetStatusBarMessage(text);
      
        return;
    }
    else 
    {
      
        QString text= omsourceVersionNo();
       
        emit SetStatusBarMessage(text);
       

    }  

 

    // ... pass it to fortran to compute their properties ...   
 
    F90MODPROC(omsource_glue, addregions)(points);
  

    delete points;  // Need to delete them  


    // refresh
    refreshListBox();
 
}
/******************************************************************
* refreshListBox
* - loads the list box that displays the parameters for each source
*****************************************************************/
void OmSourceBrowser::refreshListBox()
{
   
    // Load the sources
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources); 
    _display->toRegion(mySources);
  
  

    if (mySources.ubound() > 0)
    {
   
   
        _sourceTableBtn->setEnabled(TRUE);
	_deleteListBtn->setEnabled(TRUE);
    
     
	
        _detectSourcesBtn->setDisabled(TRUE);
        _photometryBtn->setEnabled(TRUE);
	//        _deconSourceBtn->setEnabled(TRUE);
 
	

        _spinBoxNumber->setEnabled(TRUE);
  
    
        _spinBoxNumber->setMinValue(1);
      

	_spinBoxNumber->setMaxValue(mySources.ubound());
        _spinBoxNumber->setValue(1);
        
        _group->setEnabled(TRUE);
        _viewAllBtn ->setEnabled(TRUE);
        _xCoord->setEnabled(true);
        _yCoord->setEnabled(true);
        _ascension->setEnabled(true);
        _declination->setEnabled(true);
        _magnitude->setEnabled(true);

        _viewSourceBtn->setEnabled(TRUE);
   
  

        QString string = omsourceVersionNo();
       
        emit SetStatusBarMessage(string);
        
	string.sprintf("Source Browser (No of sources=%d)", mySources.ubound());

        SetCaption(string);

        string.sprintf("Source selection (No of sources=%d)", mySources.ubound());

        
	_group->setTitle(string); 


 
    }	
    else
    {

	   
   
        _spinBoxNumber->setDisabled(TRUE);
 
    
	_deleteListBtn->setDisabled(TRUE);
        _sourceTableBtn->setDisabled(TRUE);
  
	
        _detectSourcesBtn->setEnabled(TRUE);
        _photometryBtn->setDisabled(TRUE);
	//        _deconSourceBtn->setDisabled(TRUE);
 
        _group->setDisabled(TRUE);
       _viewAllBtn ->setDisabled(TRUE);
       _deleteSourceBtn->setDisabled(TRUE);

        _xCoord->setDisabled(true);
        _yCoord->setDisabled(true);
        _ascension->setDisabled(true);
        _declination->setDisabled(true);
        _magnitude->setDisabled(true);
  
        _viewSourceBtn->setDisabled(TRUE);
 
        QString text =  
            "Please select sources manually (click on Add sources button) or click Run omdetect button to detect sources";
    
 
        emit SetStatusBarMessage(text);
   

        SetCaption("Source Browser (No of sources=0)");
      

    }  
  

}
/******************************************************
* print
******************************************************/
void OmSourceBrowser::print()
{
#ifndef QT_NO_PRINTER

    QPrinter printer; 
    if ( printer.setup(this) ) {                // opens printer dialog
        printer.setFullPage(TRUE);              // we'll set our own margins
	QPainter p;
	p.begin( &printer );                    // paint on printer
	p.setFont( p.font() );
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( &printer ); // need width/height
                                                 // of printer surface
    
        /*********************************************
	* Print what ever
	********************************************/
        (this->*Print)(&printer, &p,  &fm,  &metrics);

	p.end();                                // send job to printer
    }
#endif
}


/*****************************************************************************
*
* printHeading
****************************************************************************/
void OmSourceBrowser::printHeading(QPainter* p,  
                                  QFontMetrics* fm,  
                                  QPaintDeviceMetrics* metrics,
                                  const int MARGIN, int &yPos)
{
    QDateTime date = QDateTime::currentDateTime();  // The current date and time
    
    QString text = QString("Listing produced by ") + omsourceVersionNo()+ QString("  ") + date.toString();  
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += 2 * fm->lineSpacing();
   
    text = "TELESCOPE=" + _obsDetails.telescope + " INSTRUMENT=" + _obsDetails.instrument;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = "OBS_ID=" + _obsDetails.obsId + " EXP_ID=" + _obsDetails.expId;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = "DATE-OBS=" + _obsDetails.dateObs + " DATE-END=" + _obsDetails.dateEnd;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = "OBJECT=" + _obsDetails.object + " OBSERVER=" + _obsDetails.observer;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text.sprintf("FILTER=%s EXPOSURE=%d (SECONDS)", (const char *)_obsDetails.filter, 
           (int)_obsDetails.exposure);
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += 5 * fm->lineSpacing();

}

/************************************************************
* detectSources
* purpose:  Runs the source detection program, omdetect,
*           on the input image tro 
*************************************************************/
void OmSourceBrowser::detectSources() 
{
   
    char tempFile[100];

    TempFile::createTempFile(tempFile);
     
    // Display the dialogue to run omdetect
    DetectionDlg *detectionDlg = new DetectionDlg(this, QString(_imageFileName), QString(tempFile));

   
    int retValue = detectionDlg->exec();
 
     
    delete detectionDlg;  
    switch (retValue)
    {
        case (FAILED):
          remove(tempFile);
    
	  return;
      
        default:
          break;
    };

    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);
    const bool invoke = true;
    
    Info::Task *t;
    t = task("ommag");
    t->setParameter("set", tempFile);
  
    try {
	
        t->exec(invoke);
    }
    catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	switch( QMessageBox::warning(this, t->name(), message, "main window", "exit", QString::null, 0, 1)) 
        {
	    case 0: // Try again or Enter
              errHandler.handler(oldHandler);
	
	      return;
	    // try again
	 
	    case 1: // Quit or Escape
	    // exit
	        errHandler.handler(oldHandler);
	        Exit();
	}
      
   }
   errHandler.handler(oldHandler); // return to default error handler

 
 
    // Load the sources
   F90MODPROC(omsource_glue, loadsources)(tempFile);
   remove(tempFile);
 
    t = task("omsource");
  
    QApplication::beep();        
         
    refreshListBox();
    viewAll();
  
 
}

/************************************************************
* detectSources
* purpose:  Runs the source detection program, omdetect,
*           on the input image tro 
*************************************************************/
void OmSourceBrowser::photometry() 
{
   
     
    // Display the dialogue to run the photometry dialog
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);

    
  
  
    //   cout << "omsourceb line 255 inner radius=" << mySources[1].innerbackgroundradius << endl;
    // cout << "omsourceb line 255 outer radius=" << mySources[1].outerbackgroundradius << endl;


    if (mySources.length() < 1) return;


   // Create the photometry dialogue  
    _photometryDlg = new PhotometryDlg(_sourceNo,  
          _display, &_obsDetails, &_imageDetails, this);
   
  
   
    int retValue = _photometryDlg->exec();
    delete  _photometryDlg;
 
 
    ShowParent();
    switch (retValue)
    {
        case (QDialog::Accepted):
          _sourceListHasChanged = true;
          refreshListBox();

        case (QDialog::Rejected):

	   

        default:
          break;
    };
    viewAll();
}

/******************************************************
* getImageFile
* Purpose:- Gets the name of the input and output file names
* given as parameters
*******************************************************/

void OmSourceBrowser::getImageFile()
{
    bool hasFrameTime;

    QString filterList[4];
    
 
    filterList[0]  = "Images (*)";
    filterList[1]  = "Images (*.FIT *.fit)";
 
    filterList[2]  = "Images (*IMAGE_* *image_*)";
 
    filterList[3] = QString::null;

    while(1)
    {
    
      
        GetOldFileName(this, _imageFileName, "imageset", 
      		     "Please select the image fits file (P0*OMS*IMAGE_*FIT*)", filterList);
      


   
        DataSet *set = dataSetServer->open(string(_imageFileName), Dal::Read);
	if (set->hasAttribute("FRAMTIME"))
	    hasFrameTime = true;
	else
	    hasFrameTime = false;
         
    
        if(set->hasBlock("MODES")) 
        {
   
            dataSetServer->close(set);
 
            break;
        } 
  
 

        dataSetServer->close(set);
  
      
        QMessageBox::critical( 0, "omsource",
			       QString("The selected image file cannot be processed\n"
			          "It does not contain a 'MODES' block\n"
                                "The image file should be the product\n" 
                                "image created by ommodamp\n\n\n" 
                                 "Please try again")); 

    }  // End of while(1)
      
    setParameter("imageset", string(_imageFileName));
  

    
      if (!hasFrameTime)  // Old image file without FRAMETIME & DEADFRAC
      {
          if (stringParameter("wdxset") == "")
  	{

             QString filterList[4];
             filterList[0]  = "(*WDX*)";
             filterList[1]  = "(*wdx*)";
             
             filterList[2]  = "(*.FIT *.fit)";
             filterList[3] = QString::null;
  
             GetOldFileName(this, _wdxFileName, "wdxset", 
  		     "A window data file is needed-\n"
                     "Please select the *OM*WDX*FIT* file", filterList);
  


              setParameter("wdxset", string(_wdxFileName));

             
          }
      }
 

    
}  
/******************************************************
* getImageFile
* Purpose:- Gets the name of the input and output file names
* given as parameters
*******************************************************/

void OmSourceBrowser::getOldSrcListFile()
{


    QString filterList[4];
    filterList[0]  =  "(*)";
    
    filterList[1]  =  "(*.FIT *.fit)";
    filterList[2]  =  "SWS Files (*SWS* *sws*)";

    filterList[3] = QString::null;

  
    
    while(1) {



       GetOldFileName(this, _oldSourceListFile, "oldsrclistset", 
		       "Please select the source-list file (*P0/IO**SWS*FIT*)", filterList);

    
    

        DataSet *set = dataSetServer->open(string(_oldSourceListFile), Dal::Read);

	if (set->hasBlock("SRCLIST"))
	{

	    // Check that the observation IDs are identical
            string obsId = (set->attribute("OBS_ID")->asString()).c_str();
            dataSetServer->close(set);
	    
	    if (_obsDetails.obsId == QString((const char *)obsId.c_str()))

	        break;
            else
	      {
		QApplication::beep();
                QMessageBox::critical( this, omsourceVersionNo(),
				       QString("The observationID (") + QString(obsId.c_str()) +
                                       QString("\nin the selected file is not the same (") + 
                                       _obsDetails.obsId +
                                       QString (")\nas that in the image file\n\n\n") + 
                                       QString("Please try again")); 


              }




        }
        else
	{
        QMessageBox::critical( this, omsourceVersionNo(),
			       QString("The selected source-list file cannot be processed\n") +
			       QString("It does not contain a 'SRCLIST' table\n") +
                               QString("The source-list file should either be a product\n") +
                               QString("source-list file created by omatt\n") +
                               QString("or one that has been created by omsource\n\n\n") + 
                QString("Please try again")); 

	//l( "<font color=darkblue><u><b>"
	//          "Information menu</b></u></font>", this );

        dataSetServer->close(set);
        }
    }               
    setParameter("oldsrclistset", string(_oldSourceListFile));
}  
/****************************************************************
 *
 ***************************************************************/
bool OmSourceBrowser::getNewSrcListFile()
{
  //  cout << "getNewSrcListFile line 1361\n";
    string fileName = stringParameter("newsrclistset");
    strcpy(_newSourceListFile, fileName.c_str());
    
    //    cout << "getNewSrcListFile line 1365 _oldSourceListFile=" << _newSourceListFile << endl;

    bool result = GetNewFileName(this, _newSourceListFile, 
          "newsrclistset", "Please enter the file-name for the new source-list",
		   (const char *)omsourceVersionNo());
 
    if (!result) return false;
    setParameter("newsrclistset", string(_newSourceListFile));
 
    return true;
}
/****************************************************
* LoadImage
****************************************************/
float** LoadImage(char *fileName, int &nx, int &ny)
{
    DataSet* set = dataSetServer->open(fileName, Dal::Read);
    Array* array = set->array("PRIMARY");
    longVec arraySize = array->size();
    nx = arraySize[0];
    ny = arraySize[1];

    
    ArrayData *arrayData = array->data();
    real32 * data = arrayData->real32Data();

    float min = 1e6;
    float max = 1e-5;

    float **image = NULL;
    allocate(&image, nx, ny); 
    for (int i = 0; i < nx; i++)
        for (int j = 0; j < ny; j++)
        {
        image[i][j] =  data[i * nx + j];
        if (data[i * nx + j] < min) 
            min = data[i * nx + j];
        else
    	    if (data[i * nx + j] > max) max = data[i * nx + j];
         
    }

    delete arrayData;
    dataSetServer->close(set);
    return image;
}

/***********************************************
* setSourceCoordinates
***********************************************/
void OmSourceBrowser::setSourceCoordinates()
{
 
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    OmSourceParameter* currentSource = &mySources[_sourceNo];

 
    QString text;
    _xCoord->setText(text.sprintf("%9.1f",  currentSource->xcoord));

    _yCoord->setText(text.sprintf("%9.1f",  currentSource->ycoord));
  
    AngleFormatter af;
    af.setMode(AngleFormatter::HMS);
    _ascension->setText(af.asString(currentSource->rightascension).c_str());
    af.setMode(AngleFormatter::DMS);
    _declination->setText(af.asString(currentSource->declination).c_str());
    _magnitude->setText(text.sprintf("%5.2f+/-%3.2f",  currentSource->magnitude, currentSource->magnitudeerror) );
  

   
}
/************************************************
* updareDisplay
************************************************/
void OmSourceBrowser::updateDisplay()
{
       
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    OmSourceParameter* currentSource = &mySources[_sourceNo];
  
    _display->toRegion(mySources);
 
    _display->zoomTo(*currentSource);   
 
}
/************************************************
* diaplaySource
************************************************/
void OmSourceBrowser::displaySource()
{
       
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    OmSourceParameter* currentSource = &mySources[_sourceNo];
  

    _display->toRegion(mySources);
 
    _display->displaySource(_sourceNo, *currentSource);   
 
 

}
/**************************************************
* 
**************************************************/
void OmSourceBrowser::deconvolveSource()
{


  //    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
 

    //    DeconvolveDlg *deconvolveDlg = new DeconvolveDlg(this, _sourceNo, &mySources[_sourceNo]);
  
  //    int retValue = deconvolveDlg->exec();
  //    delete  deconvolveDlg;

  //    switch (retValue)
  //    {
  //        case (QMessageBox::Yes):
    
  //	  return;
  //       case (QMessageBox::No):

	   

  //        default:
  //          break;
  //    };

}
/*******************************************************
*
******************************************************/
void OmSourceBrowser::displayInformation()
{

    QPopupMenu *informationMenu  = new QPopupMenu( this );
    
    InformationMenu(this, informationMenu);
  
		
    informationMenu->exec(QCursor::pos());

    delete informationMenu;
   
}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::displayObservationDetails(const OmSourceBrowser::ObsDetails 
                   *obsDetails)
{
    QString text[12];
    text[0] = "Telescope=" + obsDetails->telescope;
    text[1] = "Instrument=" + obsDetails->instrument;
    text[2] = "obsId=" + obsDetails->obsId;
    text[3] = "expId=" + obsDetails->expId;
    text[4].sprintf("dateObs=%s (JD=%10.4f)", (const char *)obsDetails->dateObs, 
                      obsDetails->JDStart);
    text[5].sprintf("dateEnd=%s (JD=%10.4f)", (const char *)obsDetails->dateEnd, 
                      obsDetails->JDEnd);
    text[6].sprintf("Julian date of observation mid-point=%10.4f", 
                obsDetails->JDMiddle);
 
    text[7] = "Filter=" + obsDetails->filter;
    text[8] = "Object=" + obsDetails->object;
    text[9] = "Observer=" + obsDetails->observer;
    text[10].sprintf("Exposure=%d %s", int(obsDetails->exposure), " seconds");
  
    text[11] = QString::null;  
   
    InformationDlg *informationDlg = new InformationDlg(0, text);
  
    informationDlg->exec();
    delete  informationDlg;

}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::showObservationDetails()
{
    displayObservationDetails(&_obsDetails);

}

/*************************************************
*
************************************************/
void OmSourceBrowser::showImageDetails()
{
    displayImageDetails(&_imageDetails);

}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::displayImageDetails(const OmSourceBrowser::ImageDetails *imageDetails)
{
    QString text[7];

 
    if (imageDetails->binBpe)
        text[0] = "BinBpe=true";

    else
        text[0] = "BinBpe=false";


    text[1].sprintf("Binax1=%d", imageDetails->binAx1);
    text[2].sprintf("Binax2=%d", imageDetails->binAx2);
    text[3].sprintf("Frame time=%.1f milliseconds", imageDetails->frameTime);
    text[4].sprintf("Dead fraction=%f", imageDetails->deadFraction);
    text[5].sprintf("Image background==%f", imageDetails->background);


    text[6] = QString::null;  
   
    InformationDlg *informationDlg = new InformationDlg(0, text);
  
    informationDlg->exec();
    delete  informationDlg;

}

/*************************************************
 *
 ************************************************/
void OmSourceBrowser::okay()
{
    close();
    Exit();       
}


/*************************************************
 *
 ************************************************/
void OmSourceBrowser::restart()
{
    close();
    deleteList();
    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);
    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
 
}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::loadImage()
{
    
    close();
  
    // Get the name of the image file name
   
    addFileToList(_imageFileList, _imageFileName);
 
    if (_sourceListIsLoaded)
    {
      
      QString message("A source-list has been loaded- this will be deleted");
        switch( QMessageBox::warning(this, omsourceVersionNo(), message, "OK", "Cancel", QString::null, 0, 1)) 
        {
	    case 0: // Try again or Enter
              
	
	      break;
	    // try again
	 
	    case 1: // Quit or Escape
	    
	      return;
	}
      
    }

    deleteList();
    _sourceListIsLoaded = false;


    setParameter("imageset", "");
    setParameter("oldsrclistset", "");
    _loadImageBtn->setDisabled(true);
 
    getImageFile();
    hide();
    _reporter = new OmProgressDialog("Initialising", 18);
    
    _reporter->setProgress(1);




    if (!_display->isUp())
    {
        delete _display;
  
       _display = new OmImageDisplay;
    }  

    
    _display->displayImage(_imageFileName);
    _display->send("zoom to fit frame");

    _display->send("cmap heat"); 
   _display->send("scale zscale");
   _display->send("scale linear");
        




   
      
    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);
    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
 
    show();
  
  
    Initialise();
   
 
}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::loadSourceList()
{
    close();
    deleteList();


 // Get the name of the image file name
    setParameter("oldsrclistset", "");

    displayOldSourceListFileMenu();

    addFileToList(_oldSourceFileList, _oldSourceListFile);
     
    
    _reporter = new OmProgressDialog("Initialising", 18);
    
    _reporter->setProgress(1);
  
 

    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);
    Initialise();
   _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
 
}
/*************************************************************
*
************************************************************/
void OmSourceBrowser::mousePressEvent( QMouseEvent* event)
{
    QApplication::beep();
    if ( event->button() == RightButton )
    {   

        QPopupMenu *informationMenu  = new QPopupMenu( this );
    
        InformationMenu(this, informationMenu);
	//        informationMenu->insertItem( "&Help information",  this, 
	//              SLOT(showHelpInformation()), Qt::CTRL+Qt::Key_H );
	//        informationMenu->insertSeparator();
  
	//        informationMenu->insertItem( "&History",  this, 
	//          SLOT( showHistory()), Qt::CTRL+Qt::Key_I );
	//        informationMenu->insertSeparator();
    

	//        QPopupMenu *imageMenu  = new QPopupMenu( this, "Image file");
    
 
	//        int id = imageMenu->insertItem( "&Image file from current directory",  this, 
	//					SLOT( loadImage()));//   , Qt::CTRL+Qt::Key_I );

    //        const char * imageText = "Click this button to open an <em>image file</em>. <br><br>";
    //        imageMenu->setWhatsThis(id, imageText);
               
	
    //        int index = 1;
    //	for ( QStringList::Iterator it = _imageFileList.begin(); it != _imageFileList.end(); ++it ) {
    //            imageMenu->insertSeparator();
          
    //            QString text;
    //           text.sprintf("&%d%s", index, (const char *)*it);
	         	
    //	    imageMenu->insertItem( text,  this, SLOT (loadImage1(int)), index, index); 
    //            index++;
                     
    //	}
        
  
    //	informationMenu->insertItem("&image file", imageMenu);


    //        QPopupMenu *sourceListMenu  = new QPopupMenu( this, "Source-list file");
    
 
    //        id = sourceListMenu->insertItem( "&Source-list file from current directory",  this, 
    //          SLOT( loadImage()), Qt::CTRL+Qt::Key_I );

    //        const char * sourceListText = "Click this button to open a <em>source-list file</em>. <br><br>";
    //        imageMenu->setWhatsThis(id, sourceListText);
               
     
    //        index = 1;
    //	for ( QStringList::Iterator it = _oldSourceFileList.begin(); it != _oldSourceFileList.end(); ++it ) {
    //            sourceListMenu->insertSeparator();
     	
    //	    sourceListMenu->insertItem( *it,  this, SLOT (loadSourceList1(int)), index, index);
    //            index++; 
      
                     
    //	}
 
 

    //        informationMenu->insertItem("&source-listfile", sourceListMenu);




        informationMenu->exec(QCursor::pos() );

        delete informationMenu;
    }
  



}   
/******************************************************
*
******************************************************/
    
void OmSourceBrowser::showHelpInformation()
{

   
  //static const char* sayings[] = {
  static const QString sayings[] = {
    "<b>Saying 1:</b><br>"
    "<hr><br><br>"
    "<big>Evil is that which one believes of others.  It is a sin to believe evil "
    "of others, but it is seldom a mistake.</big><br><br>"
    "<center><i>-- H.L. Mencken</i></center>",

    "<b>Saying 2:</b><br>"
    "<hr><br><br>"
    "<big>A well-used door needs no oil on its hinges.<br>"
    "A swift-flowing steam does not grow stagnant.<br>"
    "Neither sound nor thoughts can travel through a vacuum.<br>"
    "Software rots if not used.<br><br>"
    "These are great mysteries.</big><br><br>"
    "<center><i>-- Geoffrey James, \"The Tao of Programming\"</i></center>",

    "<b>Saying 3:</b><br>"
    "<hr><br><br>"
    "<big>Show business is just like high school, except you get paid.</big><br><br>"
    "<center><i>-- Martin Mull</i></center>",

    "<b>Saying 4:</b><br>"
    "<hr><br><br>"
    "<big><b>The Least Successful Executions</b><br>"
    "<twocolumn><p>      History has furnished us with two executioners worthy of attention. "
    "The first performed in Sydney in Australia.  In 1803 three attempts were "
    "made to hang a Mr. Joseph Samuels.  On the first two of these the rope "
    "snapped, while on the third Mr. Samuels just hung there peacefully until he "
    "and everyone else got bored.  Since he had proved unsusceptible to capital "
    "punishment, he was reprieved.</p>"
    "<p>        The most important British executioner was Mr. James Berry who "
    "tried three times in 1885 to hang Mr. John Lee at Exeter Jail, but on each "
    "occasion failed to get the trap door open.<!p>"
    "<p>        In recognition of this achievement, the Home Secretary commuted "
    "Lee's sentence to \"life\" imprisonment.  He was released in 1917, emigrated "
    "to America and lived until 1933.</p></twocolumn></big><br><br>"
    "<center><i>-- Stephen Pile, \"The Book of Heroic Failures\"</i></center>",

    "<b>Saying 5:</b><br>"
    "<hr><br><br>"
    "<big>If you can, help others.  If you can't, at least don't hurt others.</big><br><br>"
    "<center><i>-- the Dalai Lama</i></center>",

    "<b>Saying 6:</b><br>"
    "<hr><br><br>"
    "<big>Television has brought back murder into the home -- where it belongs.</big><br><br>"
    "<center><i>-- Alfred Hitchcock</i></center>",

    "<b>Saying 7:</b><br>"
    "<hr><br><br>"
    "<big>I don't know who my grandfather was; I am much more concerned to know "
    "what his grandson will be.</big><br><br>"
    "<center><i>-- Abraham Lincoln</i></center>",

    0
};

    HelpDlg *helpDlg = new HelpDlg(this, sayings);    

    helpDlg->exec();
    
     
    delete helpDlg;  
 
}


/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::addFileToList(QStringList &fileList, char file[])
{  
  
 
    if (fileList.findIndex(file) != -1) return; 
  
    if (  fileList.size() > 8)  fileList.pop_front();
    //    QFileInfo fi(file);
    //    QDateTime date = fi.created(); 
   

    fileList += (QString(file)); // + " creation date=" + date.toString());
 
}

/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::saveFileNames()
{
    QSettings settings;
  
    settings.writeEntry("/OmSource/Images",  _imageFileList, QChar('\t'));
    settings.writeEntry("/OmSource/OldSourceList",  _oldSourceFileList, QChar('\t')); 
    settings.writeEntry("/OmSource/NewSourceList",  _newSourceFileList, QChar('\t'));
  
}
/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::readFileNames()
{
  
    QSettings settings;
   
    loadUsedFileNames(_imageFileList, "/OmSource/Images"); 
    loadUsedFileNames(_oldSourceFileList, "/OmSource/OldSourceList"); 
    loadUsedFileNames(_newSourceFileList, "/OmSource/NewSourceList");
 
    saveFileNames(); 
   
} 
/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::loadUsedFileNames(QStringList &list, const char *name)
{
  

    // Get the list from the system file
    QSettings settings;
    list = settings.readListEntry(name , QChar('\t'));
   
    if (list.size() == 0) return;

    QFile file; 
    QStringList temp;
    //    Check that the file still exists
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )

    {
	
        if (file.exists(*it))
            temp += (*it); 
	 

    }   
    list = temp;
} 
/*******************************************************
*
******************************************************/
void OmSourceBrowser::showHistory()
{

  //    HistoryDlg* historyDlg = new HistoryDlg(this, &_imageFileList,  
  //               &_oldSourceFileList, &_newSourceFileList);    

  //    historyDlg->exec();
         
  //    delete historyDlg;  
   
}
/*******************************************************
*
******************************************************/
void OmSourceBrowser::showSourceTable()
{
    // Load the sources 
 
  
    TableDlg* tableDlg = new TableDlg(this);    
    connect(tableDlg, SIGNAL(selectionChanged(int)), this, SLOT(  SetSpinBoxValue(int)));

    connect(tableDlg, SIGNAL(deleteSourceList()), this, SLOT (deleteList()) );
  

    tableDlg->exec();
         
    delete tableDlg;  
   
}
/*****************************************************
*
****************************************************/
void OmSourceBrowser::loadImage1(int id)
{        

    strcpy(_imageFileName,  (const char *)_imageFileList[id - 1]);
    setParameter("imageset", string(_imageFileName));
  
    getImageFile();
}
/********************************************
 *
 ********************************************/
void OmSourceBrowser::displayImageFileMenu()
{
  

    QApplication::beep();

    if (_imageFileList.size() == 0)
    {
        // Just display the file input widget      
        getImageFile();
        return;
    }

  
    // Crate a pop-up menu
    
    QPopupMenu *imageMenu  = new QPopupMenu( this, "imagemenu");
 
 
    QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
              "Please select your image file (*IMAGE_*)</b></u></font>", this);
    caption->setAlignment( Qt::AlignCenter );
    imageMenu->insertItem( caption );
  
 
    int id = imageMenu->insertItem( "&Image file from current directory",  this, 
   				    SLOT( getImageFile()));//, Qt::CTRL+Qt::Key_I );
    
    const char * imageText = "Click this button to open an <em>image file</em>. <br><br>";
    imageMenu->setWhatsThis(id, imageText);

    
    imageMenu->insertSeparator();
     	           
    // Add the image files to the menu 
    int index = 1;
    for ( QStringList::Iterator it = _imageFileList.begin(); it != _imageFileList.end(); ++it ) {
      //        imageMenu->insertSeparator();
   
        QString text;
        text.sprintf("&%d%s", index, (const char *)*it);
	
	id = imageMenu->insertItem(text,  this, SLOT (loadImage1(int)), index, index); 
	index++;

    }                  
    imageMenu->exec(QPoint(300, 300));

    delete imageMenu;
}
/************************************************************
*
***********************************************************/
void OmSourceBrowser::displayOldSourceListFileMenu()
{

    QApplication::beep();

    if (_oldSourceFileList.size() == 0)
    {
        // Just display the file input widget      
        getOldSrcListFile();
        return;
    }

  
    // Crate a pop-up menu
    
    QPopupMenu *sourceListMenu  = new QPopupMenu( this, "imagemenu");
 
 
    QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
              "Please select your source-list file (*SWS*)</b></u></font>", this);
    caption->setAlignment( Qt::AlignCenter );
    sourceListMenu->insertItem( caption );
  
 
    int id = sourceListMenu->insertItem( "&Source-list file from current directory",  this, 
   				    SLOT( getOldSrcListFile()));//, Qt::CTRL+Qt::Key_I );
    
    const char * sourceListText = "Click this button to open a <em>source-list file</em>. <br><br>";
    sourceListMenu->setWhatsThis(id, sourceListText);

    
    sourceListMenu->insertSeparator();
     	           
    // Add the image files to the menu 
    int index = 1;
    for ( QStringList::Iterator it = _oldSourceFileList.begin(); it != _oldSourceFileList.end(); ++it ) {
      //        imageMenu->insertSeparator();
   

        QString text;
        text.sprintf("&%d%s", index, (const char *)*it);
	

  	
	id = sourceListMenu->insertItem(text,  this, SLOT (loadSourceList1(int)), index, index); 
	index++;

    }                   
        
   
    sourceListMenu->exec(QPoint(300, 300));

    delete sourceListMenu;
   

}

/*****************************************************
*
****************************************************/
void OmSourceBrowser::loadSourceList1(int id)
{        
  
    strcpy(_oldSourceListFile,  (const char *)_oldSourceFileList[id - 1]);
    setParameter("oldsrclistset", string(_oldSourceListFile));
   
}



      
   


















