// -*-C++-*-
//
// File:      OmSourceBrowser.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Sep  6 11:11:03 MEST 2002
// Usage:     -
// Info:      -

#ifndef OmSourceBrowser_h
#define OmSourceBrowser_h
#include <qwidget.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qspinbox.h>
#include <qapplication.h>
// forward declarations
class OmSourceParameter;
class OmSourceEditor;
class OmImageDisplay;
class OmSourceLog;
class QListBox;
class QComboBox;
class QPushButton;
class QProgressDialog;
class OmProgressDialog;

class QSpinBox;
class OmSourceGroupBox;
class QLabel;
class OmSourceButton;
class QContextMenuEvent;
class PhotometryDlg;  
class QProcess;
class QPopupMenu;
class QStringList;

#include <iostream>

class OmSourceBrowser : public QWidget
{
  Q_OBJECT
public:
  OmSourceBrowser(QWidget* parent,const char* name=0);
  virtual ~OmSourceBrowser();
  void Initialise();
  void close(); 

private:
    static void addFileToList(QStringList &list, char file[]);
    static void loadUsedFileNames(QStringList &list, const char *name);


 
signals:
  // Emitted whenever we expect the user to add sources within the display
  //  void suspendedForAddition(); 
    
      void HideParent();
      void ShowParent();
      
      void SetCaption(QString caption);
      void SetStatusBarMessage(QString& caption, long time);
      void SetStatusBarMessage(QString& caption);
     
      void Exit();
      void Repaint(); 
      


private slots:

  
  void recompute(const OmSourceParameter& s);
  void viewAll(); // zoom out and display all sources

  void addSources(); // triggered by _addSourcesBtn
  void deleteSource(); // triggered by _editor
  void setListBoxItem(int index);
  void selectSource(int no); 
  void SetSpinBoxValue(int num);
      
  void deleteList();
  void detectSources();   
  void photometry(); 
  void deconvolveSource();   
  void okay();
  //  void quit();

  void updateDisplay();
  void displaySource();
  void showObservationDetails();  
  void showImageDetails();  
  void restart();
  void loadImage();
  void loadImage1(int id);

  void loadSourceList();
             
  //  void contextMenuEvent( QContextMenuEvent * );
  virtual void mousePressEvent( QMouseEvent* event);
  void showHelpInformation();
  void showHistory();
  void showSourceTable();  
  void displayImageFileMenu();
  void getImageFile();
  void loadSourceList1(int id);
  void displayOldSourceListFileMenu();
  bool getNewSrcListFile();
  void getOldSrcListFile();
  void displayInformation();  
 
protected:
    
    public slots:
    void selectionChanged(int num) { cout << "calling setspinboxvalue i=" << num << endl;  SetSpinBoxValue(num); };


public:
    class ObsDetails {
    public:
      QString telescope;
      QString instrument;
      QString obsId;
      QString expId;
      QString dateObs;
      QString dateEnd;
      QString filter;
      QString object;
      QString observer;
      float exposure;
      double JDStart, JDEnd, JDMiddle;
       
      ObsDetails() : telescope(""), instrument(""), obsId(""), 
           expId(""), dateObs(""), dateEnd(""), filter(""), 
	object(""), observer(""), exposure(0.0),
        JDStart(0.0), JDEnd(0.0), JDMiddle(0.0) 
           {};
     ~ObsDetails() {};
      ObsDetails(ObsDetails &obsDetails)
      {
	telescope = obsDetails.telescope;
        instrument = obsDetails.instrument;
        obsId = obsDetails.obsId;
        dateObs = obsDetails.dateObs;
        dateEnd = obsDetails.dateEnd;
        filter = obsDetails.filter;
        object = obsDetails.object;
        exposure = obsDetails.exposure;
        JDStart =  obsDetails.JDStart;
        JDEnd =  obsDetails.JDEnd;
        JDMiddle =  obsDetails.JDMiddle;



      };        

    };

  class ImageDetails {
    public:
      
      bool binBpe;
      short binAx1;
      short binAx2;
      float frameTime;
      float deadFraction; 
      float background;
      ImageDetails() :  
           binBpe(false), binAx1(1), binAx2(1),
           frameTime(0.0), deadFraction(0.0),
           background(0.0)   {};
     ~ImageDetails() {};
             

    };



  ObsDetails _obsDetails;
  ImageDetails _imageDetails;
  enum{OKAY, FAILED, INTERRUPTED, CHANGED, QUIT};

  // disabled member function
  static void displayObservationDetails(const OmSourceBrowser::ObsDetails *obsDetails);  
  static void displayImageDetails(const OmSourceBrowser::ImageDetails *imageDetails);  
  

  OmSourceBrowser(const OmSourceBrowser&);
  OmSourceBrowser& operator=(const OmSourceBrowser&);

  //
  void refreshListBox();

  void setSourceCoordinates(); 
 
  typedef void(OmSourceBrowser::*PRINT)(QPrinter* printer, QPainter* p,  
				       QFontMetrics *fm,  QPaintDeviceMetrics* metrics);

   PRINT Print;
  

  
   void printHeading(QPainter* p, QFontMetrics* fm, QPaintDeviceMetrics* metrics,
                     const int MARGIN, int &yPos);
   void print();
 
  void saveFileNames();
   void readFileNames();
  
 



  //  Info::Task *t;



  // member data
    OmSourceGroupBox*     _group;

    bool             _changed; 
    float _calApertureRadius;
  
    OmImageDisplay*  _display;  
    OmSourceEditor*  _editor;
    QProgressDialog* _progress;
    OmProgressDialog* _reporter;
   

    QSpinBox*        _spinBoxNumber; 


  //    QPushButton*     _viewAllBtn;
    OmSourceButton*    _viewAllBtn;
    OmSourceButton*     _addSourcesBtn;
    OmSourceButton*     _printLogBtn;
    OmSourceButton*     _printAveragesBtn;
    OmSourceButton*     _deleteAveragesBtn;

    OmSourceButton*     _sourceTableBtn;
 
    OmSourceButton*     _deleteListBtn;

    OmSourceButton*     _detectSourcesBtn;
    OmSourceButton*     _photometryBtn;
    OmSourceButton*     _deleteSourceBtn;
    OmSourceButton*     _viewSourceBtn;
    OmSourceButton*     _deconSourceBtn;

   
    OmSourceButton*     _exitBtn;
 
    OmSourceButton*     _cancelBtn;
    OmSourceButton*     _restartBtn;

    OmSourceButton*     _loadImageBtn;
    OmSourceButton*     _loadSourceListBtn;


    QLabel* _magnitudeLabel;


    QLineEdit* _xCoord;
    QLineEdit* _yCoord;
    QLineEdit* _ascension;
    QLineEdit* _declination;
    QLineEdit* _magnitude;
    
   

    int _sourceNo;

    char _imageFileName[256];       // For the image file name
    char _wdxFileName[256];         // For the wdx file (only needed for old SAS)

    char _oldSourceListFile[256];   // For the old source-list file name

    char _newSourceListFile[256];   // For the new source-list file name

    bool _sourceListHasChanged;
    bool _sourceListIsLoaded;     

    PhotometryDlg *_photometryDlg;  
    QProcess * _process; 
    QPopupMenu* _informationMenu;
       
  // The following are to store the image/source-list files accessed
    QStringList _imageFileList;
    QStringList _oldSourceFileList;
    QStringList _newSourceFileList;

 
  
 };

#endif


