#include "OmSourceDeconvolveDlg.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include <qstring.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <ExceptionErrorHandler.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdatetime.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#include "OmImageDisplay.h"
#include  "OmSourceParameters.h"

F90MODPROCPROTO(void, omsource_glue, deconvolve)
   (LsqData *data,  OmSourceParameter  *source);

/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/
 
DeconvolveDlg::DeconvolveDlg(QWidget *parent, int no, OmSourceParameter *source) :
  QDialog(parent, "", true), _source(source), _noSources(1), 
  _sourceIndex(no), _sourceNo(0), _status(0)


 


  
{
  
    setCaption(omsourceVersionNo());
    setGeometry(0, 0, 400, 600);
    _label = new QLabel(this);
    _label->setGeometry(10, 10, 350, 30);
    cout << "DeconvolveDlg xpos=" << _source->xcoord << " " << _source->ycoord << endl;

    int height = 25;
    {
    QGroupBox *group = new QGroupBox(this);
   
    group->setGeometry(50, 100, 150, 50);
 
    QLabel* label = new QLabel(group);
    label->setGeometry(10, 10, 80, height);
    label->setText("No of sources");
    _noSourcesSpinBox = new QSpinBox(group, " ");
    _noSourcesSpinBox->setGeometry(100, 10, 50, height);
    _noSourcesSpinBox->setRange(2, 4);
    _noSourcesSpinBox->setValue(2);
    connect(_noSourcesSpinBox, SIGNAL(valueChanged(int)), this, SLOT(  setNoSources(int)));

    }

    // Parameter buttons



    {   
      int x = 10;
      int y = 10;
    QGroupBox *group = new QGroupBox(this, "Background parameters");
    //    group->setText("Intitial source parameters");   
    group->setGeometry(50, 200, 250, 80);

    
    QLabel* label = new QLabel(group);
    label->setGeometry(x, y, 150, height);
    label->setText("Background pixel count");
    _backgroundEdit = new QLineEdit(group);
    _backgroundEdit ->setGeometry(x + 170, y, 50, height);

     connect(_backgroundEdit, SIGNAL(textChanged(const QString &)), this, SLOT(backgroundChanged(const QString &)));
    _backgroundCheck = new QCheckBox("Fixed", group);
    _backgroundCheck ->setGeometry(x + 200, y, 50, height);
    connect(_backgroundCheck, SIGNAL(clicked()), this, SLOT(fixParameterBackground()));
   
  
  


    }
    {
    int y = 10;
    int x = 10;

    int dy = 35;
    QGroupBox *group = new QGroupBox(this, "Individual source parameters");
    //    group->setText("Intitial source parameters");   
    group->setGeometry(50, 300, 250, 200);

    _sourceNoSpinBox = new QSpinBox(group, " ");
    _sourceNoSpinBox->setGeometry(50, 10, 50, height);
    _sourceNoSpinBox->setRange(1, 3);
    _sourceNoSpinBox->setValue(1);
    connect(_sourceNoSpinBox, SIGNAL(valueChanged(int)), this, SLOT(  setSourceParameters(int)));

    y += dy;
 
    QLabel* label = new QLabel(group);
    label->setGeometry(x, y, 80, height);
    label->setText("X coord");
    _xPosEdit = new QLineEdit(group);
    _xPosEdit ->setGeometry(x + 80, y, 50, height);
    

    connect(_xPosEdit, SIGNAL(textChanged(const QString &)), this, SLOT(xPosChanged(const QString &)));
    _xPosCheck = new QCheckBox("Fixed", group);
    _xPosCheck ->setGeometry(x + 150, y, 50, height);
    connect(_xPosCheck, SIGNAL(clicked()), this, SLOT(fixParameterXpos()));
 

    y += dy;
    QLabel* label1 = new QLabel(group);
    label1->setGeometry(x, y, 80, height);
    label1->setText("Y coord");
    _yPosEdit = new QLineEdit(group);
 
    _yPosEdit ->setGeometry(x + 80, y, 50, height);
    connect(_yPosEdit, SIGNAL(textChanged(const QString &)), this, SLOT(yPosChanged(const QString &)));
    _yPosCheck = new QCheckBox("Fixed", group);
    _yPosCheck ->setGeometry(x + 150, y, 50, height);
    connect(_yPosCheck, SIGNAL(clicked()), this, SLOT(fixParameterYpos()));
 

    y += dy;
    QLabel* label2 = new QLabel(group);
    label2->setGeometry(x, y, 80, 30);
    label2->setText("Central intensity");
     _i0Edit = new QLineEdit(group);
 
    _i0Edit ->setGeometry(x + 80, y, 50, height);
    connect(_i0Edit, SIGNAL(textChanged(const QString &)), this, SLOT(i0Changed(const QString &)));
    _i0Check = new QCheckBox("Fixed", group);
    _i0Check ->setGeometry(x + 150, y, 50, height);
    connect(_i0Check, SIGNAL(clicked()), this, SLOT(fixParameterI0()));
 
  
    }


    // Compute button
    int yPos = this->height() - 80;
    int xPos = this->width() / 2;
  
    _computeButton = new QPushButton( this, "PushButton1" );
    _computeButton->setGeometry(xPos, yPos, 70, 35); 
    _computeButton->setText( tr( "Compute" ) );
    connect(_computeButton, SIGNAL(clicked()), this, SLOT(compute()));
    QToolTip::add( _computeButton , 
            "Click here to determine the model parameters");
 
   


   // Quit buttons
    
    yPos = this->height() - 50;
    xPos = this->width() / 2;
    _quitButton = new QPushButton( this, "PushButton1" );
    _quitButton->setGeometry(xPos, yPos, 70, 35); 
    _quitButton->setText( tr( "Quit" ) );
    connect(_quitButton, SIGNAL(clicked()), this, SLOT(quit()));
    QToolTip::add( _quitButton , 
            "Click here to return to the main window");
 
   
    initialise();    
   
   
    
};
/*********************************************
* quit
*********************************************/

void DeconvolveDlg::quit()
{


    //   OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
 
  //    F90MODPROC(omsource_glue, replace)(_sourceNo, _originalSource);
    
  

    //  mySources[_sourceNo] = *_originalSource;
    reject();


}
/*****************************************************
*
****************************************************/
void DeconvolveDlg::setSourceParameters(int no)
{
   _sourceNo = no - 1;
   
}
void DeconvolveDlg::setNoSources(int no)
{
  _data.noStars = no;
   
}
/*****************************************************
*
****************************************************/
void DeconvolveDlg::xPosChanged(const QString &text)
{
  
   _data.xPos[_sourceNo] =  getValue(text, 0.0, 0.0);
}
/*****************************************************
*
****************************************************/
void DeconvolveDlg::yPosChanged(const QString &text)
{
  


   _data.yPos[_sourceNo] =  getValue(text, 0.0, 0.0);
}
/*****************************************************
*
****************************************************/
void DeconvolveDlg::i0Changed(const QString &text)
{
   


 _data.i0[_sourceNo] = getValue(text, 0.0, 0.0);
}
/*****************************************************
*
****************************************************/
void DeconvolveDlg::backgroundChanged(const QString &text)
{
   


   _data.backGround = getValue(text, 0.0, 0.0);

}
/*****************************************************
 *
 ****************************************************/
void DeconvolveDlg::fixParameterBackground()
{
  _data.toFit[0] = !_data.toFit[0];


} 

/*****************************************************
 *
 ****************************************************/
void DeconvolveDlg::fixParameterXpos()
{
     int no = _sourceNo * 3 + 1;
     _data.toFit[no] = !_data.toFit[no];


} 

/*****************************************************
 *
 ****************************************************/
void DeconvolveDlg::fixParameterYpos()
{
     int no = _sourceNo * 3 + 2;
     _data.toFit[no] = !_data.toFit[no];

} 
/*****************************************************
 *
 ****************************************************/
void DeconvolveDlg::fixParameterI0()
{

     int no = (_sourceNo + 1) * 3;
     _data.toFit[no] = !_data.toFit[no];


} 







/*****************************************************
 *
 ****************************************************/
void DeconvolveDlg::compute()
{
  cout << "In function compute line 251, source index =" << _sourceIndex << endl;
    _computeButton->setDisabled(true);
    _data.noStars = 1;
    _data.backGround = 100.0;
    cout << "xpos=" << _data.xPos[0] << " ypos=" << _data.yPos[0] << " i0=" << _data.i0[0] << endl; 
    cout << "Source centre=" << _source->xcoord << " " << _source->ycoord << endl;
   
    F90MODPROC(omsource_glue, deconvolve)(&_data, _source);
    _status = _data.exitValue;    
   cout << "Source centre=" << _source->xcoord << " " << _source->ycoord << endl;
 
    cout << "xpos=" << _data.xPos[0] << " ypos=" << _data.yPos[0] << " i0=" << _data.i0[0] << endl;  
    _computeButton->setEnabled(true);
} 
/*****************************************************
 *
 ****************************************************/
double DeconvolveDlg::getValue(const QString &text, double min, double max)
{
   char *endPtr;
   double value = strtod(text, &endPtr);
   if (endPtr)
   {
   }  
   

   if (value < min || value > max)
     {



     }
   return value;
}
void DeconvolveDlg::initialise()
{
  _data.noStars = 1;
 for (int i = 0; i < 10; i++)
   {
   _data.toFit[i] = true;
   } 

 // return;  
  cout << " DeconvolveDlg line 290\n";
    QString text;
    _backgroundEdit->setText("1.0");
    _backgroundCheck->setChecked(false);

  cout << " DeconvolveDlg line 296\n";
 
  cout <<  _source->xcoord << endl;
    text.sprintf("%7.2f", _source->xcoord); 
    _xPosEdit->setText(text);
    _xPosCheck->setChecked(false);

  cout << " DeconvolveDlg line 302\n";
 
  // NB DOESNT LIKE THE FOLLOWING LINE
    text.sprintf("%7.2f", _source->ycoord);  
    _yPosEdit->setText(text);
    _yPosCheck->setChecked(false);
 
    cout << " DeconvolveDlg line 309\n";

    //     text.sprintf("%7.1f", _source->i0);  
  
    //    _i0Edit->setText(text);;
    _i0Check->setChecked(false);





 cout << " DeconvolveDlg line 308 \n";
 


}



