#include "OmSourceDetectDialog.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include <qstring.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <omdetect.h>
#include <ExceptionErrorHandler.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdatetime.h>
#include <qprocess.h>
#include <qtextedit.h>
#include <iostream>
#include <qcombobox.h>
#include <qfont.h>
#include <qgroupbox.h>
#include <qlayout.h>

#include "OmSourceButton.h"
#include "OmSourceLabel.h"
#include  "OmSourceGroupBox.h"
/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/

DetectionDlg::DetectionDlg(QWidget *parent, QString imageFile, QString sourceListFile) :
  QDialog(parent, "", false), _imageFile(imageFile), _sourceListFile(sourceListFile)
{

  
    setCaption(omsourceVersionNo());



  

   // create the image buttons
    _label = new OmSourceLabel(this, "", 10, 10, 450, 50 );
  
   
    _label->setAlignment(Qt::AlignCenter);
 
    _label->setEnabled(true);
  
    _group = new OmSourceGroupBox( this, "omdetect parameters- please change if you wish",
               10, 100, 400, 120);
      
 
  _cpuLabel = new OmSourceLabel(this, "CPU time=0 seconds", 0,  500, height);
  _cpuLabel->setEnabled(true);


  
    _group->setTitle("omdetect parameters- please change if you wish");

    int xPos = 20;
    int yPos = 20;
    int height = 25; 



    _label1 = new OmSourceLabel(_group, "nsigma", xPos,  yPos, height, 
                              QString("parameter nsigma (pixel values with a value >"
                              "= background mode + nsigma * sqrt(background mode)"
                              "will be regarded as being part of a possible source"));


    _spinBox1 = new QSpinBox(_group, "a");


    _spinBox1->setGeometry(xPos + 100, yPos, 60, height);
 
 
    _spinBox1->setMinValue(1);
    _spinBox1->setMaxValue(20);
    _nSigma = 2;
    _spinBox1->setValue(_nSigma);
   
 
    QToolTip::add( _spinBox1, "Change the parameter nsigma (1-");

    connect(_spinBox1, SIGNAL(valueChanged(int)), this, SLOT(  setnSigma(int)));

    _spinBox1->show();


     yPos += 30;
 

     _label2 = new OmSourceLabel(_group, "minsignificance", xPos, yPos, height,
              "parameter minsignificance (source counts / sqrt(background counts");





    _spinBox2 = new QSpinBox(_group, "a");


    _spinBox2->setGeometry(xPos + 100, yPos, 60, height);
 
 
    _spinBox2->setMinValue(1);
    _spinBox2->setMaxValue(20);
    _minSignificance = 3;
    _spinBox2->setValue(_minSignificance);
   
 
    QToolTip::add( _spinBox2, "Change the parameter minsignificance (1-");

    connect(_spinBox2, SIGNAL(valueChanged(int)), this, SLOT(  setminSignificance(int)));

    _spinBox2->show();

   

    yPos += 30;
    _detectExtendedCheck = new QCheckBox("Detect extended sources", _group);
    _detectExtendedCheck->setGeometry(xPos, yPos, 160,  height);
    _detectExtendedCheck->setChecked(TRUE);

    QToolTip::add(_detectExtendedCheck, "If checked, omdetect will look for extended sources");



    QGroupBox *group = new OmSourceGroupBox( this, "", 20, 250, 200, 80);   
   
    
  


    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
   

  

    _terminateBtn = new OmSourceButton(group, "Terminate");
    _terminateBtn->setDisabled(true);
    layout->addWidget( _terminateBtn);
 
   
    QObject::connect(_terminateBtn, SIGNAL(clicked()), this, SLOT(terminate()));
  

    _computeBtn = new OmSourceButton(group, "Compute");
   
    
    layout->addWidget( _computeBtn);
   
    QObject::connect(_computeBtn, SIGNAL(clicked()), this, SLOT(runOmdetect()));

  


    _omdetectComboBox = new QComboBox(false, this);
    _omdetectComboBox->setGeometry(10, 350, 600, 50);
    _omdetectComboBox->show();    



   _quitBtn = new OmSourceButton(this, "Quit", 250, 420, 120, 30);
   _quitBtn->adjustSize();
   _quitBtn->setEnabled(true);
    
   
    QObject::connect(_quitBtn, SIGNAL(clicked()), this, SLOT(quit()));
  




};
/*******************************************************************
* NB Need this routine to allow the widget changes to be made
*
*******************************************************************/
void DetectionDlg::runOmdetect()
{
    Info::Task *t;
    t = task("omdetect");
   

    QDateTime date = QDateTime::currentDateTime();
  
    QString text; 
        text.sprintf("Running omdetect, version %s, %s", 
                   t->version(), (const char*)date.toString());
	//     QFont serifFont( "Times", 20, QFont::Bold );
   
	//    _label->setFont(serifFont);
   
    _label->setText(text);
     
    _spinBox1 -> setDisabled(TRUE);
    _spinBox2 -> setDisabled(TRUE);
    _detectExtendedCheck->setDisabled(TRUE);

    repaint();
    qApp->processEvents();
  

     
  _process = new QProcess();
  _process->addArgument("omdetect");
  _process->addArgument("set="+_imageFile);
  _process->addArgument("outset="+_sourceListFile);
  _interupted = false;
   
   text.sprintf("nsigma=%f", (float)_nSigma);
  _process->addArgument(text);

   text.sprintf("minsignificance=%f", (float)_minSignificance);
  _process->addArgument(text);

  if (_detectExtendedCheck->isChecked() )
      _process->addArgument("detectextended=t");
  else
     _process->addArgument("detectextended=f");
 
    


   QObject::connect( _process, SIGNAL(processExited()), this, SLOT(finished()));
   QObject::connect( _process, SIGNAL(readyReadStdout()), this, SLOT(slotDataOnStdout()));
   
 
   _computeBtn->setDisabled(true); 
   _omdetectComboBox->clear();
  _quitBtn->setDisabled(true);
 
    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);
    exit(0);  
   try {
	
   if ( !_process->start())
   {   




      QMessageBox::warning(this, "omdetect", "failure", "main window", "exit", QString::null, 0, 1); 
	
   }
   else
   {
    _timer = new QTimer( this);

    connect( _timer, SIGNAL(timeout()), this, SLOT(updateCpu()) );
    _cpuTime.start();
  
    _timer->start( 1000); // 2 seconds single-shot timer
     

   }



   }
   catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	QMessageBox::warning(this, "omsource", message, "main window", "exit", QString::null, 0, 1); 
	done(OmSourceBrowser::FAILED);
      
   }
   errHandler.handler(oldHandler); // return to default error handler
  


  _terminateBtn->setEnabled(true);
  _quitBtn->setDisabled(true);

 
    
}
/***********************************************
*
***********************************************/
void DetectionDlg::finished()
{
  delete _process;

  delete _timer;   
     
   _terminateBtn->setDisabled(true);
   _label->setText(" ");
   if (_interupted) return;

   QMessageBox::information( this, "omsource",
    "omdetect has now finished running.\n"
    "ommag will now run." );



   done(OmSourceBrowser::OKAY);
  
}
/***********************************************
*
***********************************************/
void DetectionDlg::terminate()
{
  _process->tryTerminate();
  _terminateBtn->setDisabled(true);
  _quitBtn->setEnabled(true);
  _computeBtn->setEnabled(true); 
 
  QTimer::singleShot(5000, _process, SLOT(kill()));

  _interupted = true;  
  _label->setText(" ");
    
    _spinBox1 -> setEnabled(TRUE);
    _spinBox2 -> setEnabled(TRUE);
    _detectExtendedCheck->setEnabled(TRUE);


    delete _timer;   
  
    _group->setEnabled(true);

}
/***********************************************
*
***********************************************/

void DetectionDlg::slotDataOnStdout()
{
  while( _process->canReadLineStdout() ) {
    QString line = _process->readLineStdout();
 
    _omdetectComboBox->insertItem(line);
    _omdetectComboBox->setCurrentText(line);
   
  }
}
void DetectionDlg::updateCpu()
{
    QApplication::beep();
    QString text;
    text.sprintf("Elapsed time=%d", _cpuTime.elapsed());
    cpuLabel.setText(text);

    Info::Task *t;
    t = task("omdetect");
   

    QDateTime date = QDateTime::currentDateTime();
   
    text.sprintf("Running omdetect, version %s, %s", 
       t->version(), (const char*)date.toString());
       
   
    _label->setText(text);


}







