// -*-C++-*-
//
// File:      OmSourceLog.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Wed Oct  2 15:31:02 CEST 2002
// Usage:     -
// Info:      -

#include "OmSourceLog.h"
#include "OmSourceParameters.h"
#include <qstring.h>
#include <qtooltip.h>
#include <qheader.h>
/*****************************************************
*
*****************************************************/
OmSourceLog::OmSourceLog(QWidget* parent, const char* name) : QTable(parent, name)
{
    int noColumns = 7;

    setReadOnly(true);   // Can only read the table
    setSelectionMode(QTable::SingleRow);
    setNumCols( noColumns);
    setNumRows(1);

    
    QHeader *th = this->horizontalHeader();
 
    th->setLabel(0,"Aperture");
 

    th->setLabel(1,"Inner Bgr");
 
    th->setLabel(2,"Outer Bgr");


    th->setLabel(3,"Axial ratio");
 

    th->setLabel(4,"Position angle");
 

  

    th->setLabel(5,"Magnitude");
 

    th->setLabel(6,"  +/-");
 

    setText(0, 0,"Aperture");

    setText(0, 1,"Inner Bgr");
 
    setText(0, 2,"Outer Bgr");
  
    setText(0, 3,"Axial ratio");
 

    setText(0, 4,"Position angle");
 

    setText(0, 5,"Magnitude");
 

    setText(0, 6," +/-");
 

        
 

    /********************************************
    * Adjust the size of the cells
    *******************************************/
    for (int i = 0; i < noColumns; i++)
    {   
      
        setColumnWidth(i, text(0, i).length() * 7);


    }
    setNumRows(0);
    
}
/*****************************************************
*
*****************************************************/
OmSourceLog::~OmSourceLog()
{
}
/*****************************************************
*
*****************************************************/
void OmSourceLog::add(const OmSourceParameter& s)
{
    int rowNo = numRows();

    setNumRows(rowNo + 1);
    int colNo = 0;

    QString text;
   
					

    if (s.photometryradius == 0.0)

        setText(rowNo, colNo++, "extended");
             

    else
        setText(rowNo, colNo++, text.sprintf("%2d", int(s.photometryradius)));
    



    setText(rowNo, colNo++, text.sprintf("%2d", int(s.innerbackgroundradius)));
    setText(rowNo, colNo++, text.sprintf("%2d", int(s.outerbackgroundradius)));
    setText(rowNo, colNo++, text.sprintf("%2d", int(s.axialRatio)));
    setText(rowNo, colNo++, text.sprintf("%3d", int(s.alpha)));


    setText(rowNo, colNo++,  text.sprintf("%5.2f", s.magnitude));
    setText(rowNo, colNo++,  text.sprintf("%5.2f", s.magnitudeerror));






     QToolTip::add( this , "This is the source-list table\n"
                             "It stores various paramters for each source\n"
                             "Column 1 = aperture radius\n"
                             "Column 2 = inner background annulus radius\n"
                             "Column 3 = outer background annulus radius\n"
                             "Column 4 = Axial ratio of ellipse\n"
                                         "(semi-minor axis of ellipse=aperture radius\n"
                                         "semi-major axis = semi-minor axis x axial ratio)"
                             "Column 5 = Position angle of ellipse\n"
                                         "(measured clockwise from x-origin)\n"
                             "Column 6 = instrumental magnitude\n"
                             "Column 7 = instrumental magnitude error");



}
/*****************************************************
*
*****************************************************/
QString OmSourceLog::getLine(const int index)
{
    QString line;
    for (int i = 0; i < numCols(); i++)
        line += text(index, i) + " ";

    return line;
}
/*****************************************************
*
*****************************************************/
void OmSourceLog::restart()
{
    setNumRows(0);
}
















