// -*-C++-*-
//
// File:      OmSourceRegions.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Sep 10 10:58:50 CEST 2002
// Usage:     interface of C++/f90
// Info:      
#include "OmSourceRegions.h"
#include "DopeVector.h"
#include "OmImageDisplay.h"
#include <cmath>
#include <iostream>


OmSourceRegions* OmImageRegions::_oldRegions = 0;
OmSourceRegions* OmImageRegions::_backgroundRegions = 0;

int OmImageRegions::_noBackgroundRegions = 0;
int OmImageRegions::_noRegions = 0;

/****************************************************************
*
***************************************************************/
OmSourceRegions* OmImageRegions::GetRegions(OmImageDisplay*  _display) 
{
   
    GeoRegion gReg = _display->regionSelection(ImageDisplayer::IMAGE, "yellow");
    //    cout << "Getregions line 31\n"; 
 
    // count the number of selected points
    int nPoints = 0;
   
    {
    for(vector<Component>::const_iterator it = gReg.begin();
	it != gReg.end(); it++)
      {  
        nPoints += (*it).size();
     
      }
    }
    //    cout << "Getregions line 44 - nPoints=" << nPoints << endl; 
  
       if (nPoints < 1) return 0;

	//    if (nPoints < 1)
	//      return  new OmSourceRegions(0, -1);
 
    // now we contruct a dopevector...

    OmSourceRegions* points = new OmSourceRegions(1, nPoints);
    
 
    unsigned long idx = 1;
 
   
    for(vector<Component>::const_iterator it = gReg.begin();
	it != gReg.end(); it++) { // should be one only....
        const Component& component = (*it);
        for(vector<FShape*>::const_iterator ic = component.begin();
	    ic != component.end(); ic++) 
	  {

	      (*points)[idx++] = (*ic)->refPoint();
	  }
    }
 
 
   
    return points;    // The selected points  
 
}  

/**********************************************************************
 *
 *********************************************************************/
OmSourceRegions* OmImageRegions::GetNewRegions(OmImageDisplay*  _display) 
{
     
    // Get all the regions from the image
    OmSourceRegions* regions = GetRegions(_display);
    if (regions == 0) return 0;
    //    cout << "OmSourceRegions.cc GetNewRegions line 83\n";
  
    int noPoints = regions->ubound(); 
    //cout << "OmSourceRegions.cc GetNewRegions line 86 noPoints=" << noPoints << endl;
   

    if (noPoints <= 0) 
    {
      if (_noRegions > 0)
          return _oldRegions;
      else
	{
       
	    return 0; 
	  
	}
    }
      

    if (_noRegions == 0) 
    {


         _oldRegions = new OmSourceRegions(regions);
 

         _noRegions = noPoints;
     
         return regions;

    }

    // Compare the regions with the old ones
    float tolerance = 5.0;

    int *keep = new int[noPoints + 1];
    int nToKeep = noPoints; 
  
    // populate the 'keep' and count the unique points
   
    for (int i = regions->lbound(); i <= regions->ubound(); i++)
    {
        keep[i] = true;
	for(int j = _oldRegions->lbound(); j <= _oldRegions->ubound(); j++)  
	{
		   
	  float dx = (*regions)[i].x() - (*_oldRegions)[j].x();
           
	  float dy = (*regions)[i].y() - (*_oldRegions)[j].y();
	  float radius = sqrt(dx * dx + dy * dy);
         
            
	  if (radius <= tolerance)
	  {
	      keep[i] = false;
	      nToKeep = nToKeep - 1;
	      break;
	  }
	}
    }



    if (nToKeep ==  0) 
    {
      delete [] keep;
            return 0;
	    //  return  new OmSourceRegions(0, -1);
 
    }
    // now we create an index of all regions that are new
    
    OmSourceRegions* tempPoints = new OmSourceRegions(1, nToKeep);
  
     
    int j = 1;
   
    for(int i = regions->lbound(); i <= regions->ubound(); i++)  
    {
        
        if (keep[i]) 
        {
	  
	    (*tempPoints)[j++] = (*regions)[i];

	}
    }
    delete [] keep; 
    
    // Replace the old regions with the new ones
   
    if (_noRegions > 0) delete _oldRegions;
    _oldRegions = new OmSourceRegions(regions);
  
    delete regions; 
    _noRegions = noPoints;


    return tempPoints;  // The selected points  
 
}  
/************************************************************************
 *
 ***********************************************************************/
void  OmImageRegions::updateBackgroundRegions(OmSourceRegions *regions)
{

   
    int size1 = regions->ubound();
    if (size1 <= 0) return;
   

    if (_noBackgroundRegions <= 0)
    {
       
      _backgroundRegions = new OmSourceRegions(regions);
      _noBackgroundRegions = size1;      
       return;

    }
    int size2 = _backgroundRegions->ubound();
 
              
    int size3 = size1 + size2;
    OmSourceRegions* tempRegions = new OmSourceRegions(1, size3);

    for (int i =  _backgroundRegions->lbound(); i <=  _backgroundRegions->ubound(); i++)
    {
      (*tempRegions)[i] =   (*_backgroundRegions)[i];

    }
    for (int i =  regions->lbound(); i <=  regions->ubound(); i++)
    {
      (*tempRegions)[i + size2] =   (*regions)[i];

    }
    delete _backgroundRegions;
    _backgroundRegions = new OmSourceRegions(tempRegions);
    _noBackgroundRegions = size3;      
  
    delete tempRegions;
}





