! -*-f90-*-
!
! Author:    jbakkeretN
! Generated: Wed Aug  9 17:30:51 MEST 2002
! Info:      interface for GUI c++code
! Note:
!  * The GUI should always call "init()" first, and "finish()" last.
!  * In comments, I usually talk about methods. A method may be either a
!    function OR subroutine within a module
!  * Any variable starting with 'my' is global to this module
!  * Please read the comments below ;-)

module omsource_glue
   use param
   use sla
   use omsource_old
   use omsourceparameter_mod
   use omsourceparam_mod

   use omsourceregion_mod
   use ommagnitude_mod
   use omprogress_mod
!   use omsourcestardecon_mod
   use omskybackground_mod
   use omphotometry_mod   ! In mssllib for doing aperture photometry

   implicit none
   
   type(SourceParameters), pointer :: mySources(:) => null()           ! The global source-list
   type(OmSourceRegion), pointer ::   mySourceRegions(:) => null() !   The globalbackground source-list

   type(SourceParameters), pointer :: myOldSources(:) => null()        ! The old global source-list


   type(SourceParameters), pointer :: myBackgroundSources(:) => null() ! The globalbackground source-list
   type(OmSourceRegion), pointer ::   myBackgroundRegions(:) => null() ! The globalbackground source-list


   type(SourceParameters), pointer :: myAddedSources(:) => null()           ! The sources that the 
                                                                            ! user has selected


   type(SourceParameters), pointer :: myAddedBackgroundSources(:) => null() ! The background sources 

                                                                            !that the user has landed
!   real(single), pointer :: myImage(:, :) = null()

   


   real ::  mycalapertureradius

   type(PhotometryParameters) :: myPhotometryParameters
contains

   ! Info: This is always the first function that is called by the GUI;
   !       in here we initialise/load/whatever




subroutine init(loaded)
    logical(boolean), intent(out) :: loaded
    
    
    call omsource_init(mySources, myBackgroundSources, &
                       myAddedSources, myAddedBackgroundSources, &

                       myPhotometryParameters) 

    loaded = associated(mySources)
end subroutine init

 


   ! Info: If the GUI closes, it calls
 
subroutine finish()
     
    call omsource_finish(mySources, myOldSources, myBackgroundSources, &
                       myAddedSources, myAddedBackgroundSources)
 
end subroutine finish


subroutine loadimage(image)
real(single), pointer :: image(:, :)

    image=>myImage
 
end subroutine loadimage


! Subroutine for loading the sources from the source-list file
! produced by omdetect in omsourcebrower::SourceDetection
subroutine loadsources(fileName)
    character(len=*), intent(in) :: fileName
    call omsource_loadsources(mySources, fileName)
end subroutine loadsources





   ! Info: Recompute a single source as aperture, and background radii
   !       may have changed
subroutine recompute(id)
    integer, intent(in) :: id
      
    call omsource_recompute(mySources, id)
end subroutine recompute
        




!************************************************
! Deconvolves a source into 2 or more sources
!***********************************************
subroutine deconvolve(dataIn, source)
    type(LsqData), intent(inout) :: dataIn
    type(SourceParameters), intent(inout) :: source
   
 
      
!    call stardecon_deconvolve(myImageWindow%bx,  myImageWindow%by, &
!             dataIn, source, myImage)
   
 
end subroutine deconvolve


   ! Info: Remove a single source at location id
subroutine remove(id)
    integer, intent(in) :: id
      
    call omsource_delete(mySources, id)
end subroutine remove

! Info: deletes the whole source list
subroutine deleteList()
    call omsource_deleteList(mySources)
end subroutine deleteList


!**********************************************
! Replaces the new list with the old one
!*********************************************
subroutine restoreOldList()

    
    if (associated(mySources) .and. associated(myOldSources)) then
       

        call delete(mySources) 
        call new(mySources, low = 1, high = size(myOldSources))
 
      
        mySources = myOldSources

    end if
    
end subroutine restoreOldList

!**********************************************
! Replaces the new list with the old one
!*********************************************
subroutine saveOldList()

    if (associated(myOldSources)) then
        call delete(myOldSources)
    end if

    if (associated(mySources)) then
  
        call new(myOldSources, low = 1, high = size(mySources))

         myOldSources = mySources
     
 
    end if
    
   
end subroutine saveOldList

!********************************************
! Replaces a source
!*******************************************
subroutine replace(id, source)
  
    integer, intent(in) :: id
    type(SourceParameters), intent(in) :: source
   
    mySources(id) = source
!    write(*, *) "omsource_glue line 196 replaceid=", id,&
!          " source%rawbackgroundcountrate=", mySources(id)%rawbackgroundcountrate
end subroutine replace    


   ! Info: User has selected regions in the image; now it is up to us to:
   !       - identify regions that are not found in the sourcelist already
   !       - enlarge the sourcelist with information within the regions
   !       - compute the source properties for the added sources
subroutine addRegions(regions, radius)
    type(OmSourceRegion),intent(in) :: regions(:)
    real(single), intent(in) :: radius

    call omsource_add(mySources, regions, radius, myAddedSources)
   
end subroutine addRegions



subroutine addRegions1(regions)
    type(OmSourceRegion),intent(in) :: regions(:)
  
    call omsource_add1(mySources, regions, myAddedSources)

   
end subroutine addRegions1





subroutine addBackgroundRegions(regions, radius)
    type(OmSourceRegion), intent(inout) :: regions(:)
    real(single), intent(in) :: radius
    call omsource_addBackgroundSources(mySources, myBackgroundSources, regions, radius)
end subroutine addBackgroundRegions



!***************************************************
!
!
!***************************************************
subroutine AppendToSourceRegions(regions)
    type(OmSourceRegion), intent(in) :: regions(:)
    type(OmSourceRegion), pointer :: tempRegions(:) => null()
    integer :: size1, size2 , size3   
    


    if (.not. associated(mySourceRegions)) then
        call new(mySourceRegions, 1, ubound(regions, 1))
        mySourceRegions = regions



        return
    end if
   


  
    size1 = ubound(regions, 1);
    if (size1 .eq. 0) return;
   
  
    size2 = ubound(mySourceRegions, 1)

    size3 = size1 + size2
    call new(tempRegions, 1, size2)
    

    tempRegions = mySourceRegions
    

    call delete(mySourceRegions)
    call new (mySourceRegions, 1, size3)



    mySourceRegions(1 : size2) = tempRegions
 
    call delete(tempRegions)

    ! The new background regions
    mySourceRegions(size2 + 1 : size3) = regions

  
end subroutine AppendToSourceRegions



!***************************************************
!
!
!***************************************************
subroutine AppendToBackgroundRegions(regions)
    type(OmSourceRegion), intent(in) :: regions(:)
    type(OmSourceRegion), pointer :: tempRegions(:) => null()
    integer :: size1, size2 , size3   
   


    if (.not. associated(myBackgroundRegions)) then
        call new(myBackgroundRegions, 1, ubound(regions, 1))
        myBackgroundRegions = regions



        return
    end if
   
  
    size1 = ubound(regions, 1);
    if (size1 .eq. 0) return;
   
  
    size2 = ubound(myBackgroundRegions, 1)

    size3 = size1 + size2
    call new(tempRegions, 1, size2)
    

    tempRegions = myBackgroundRegions
    

    call delete(myBackgroundRegions)
    call new (myBackgroundRegions, 1, size3)



    myBackgroundRegions(1 : size2) = tempRegions
 
    call delete(tempRegions)

    ! The new background regions
    myBackgroundRegions(size2 + 1 : size3) = regions

  
end subroutine AppendToBackgroundRegions
!***************************************************
!
!
!***************************************************

subroutine getBackgroundRegions(regions)
    type(OmSourceRegion),intent(in) :: regions(:)


    call omsource_getnewsources(mySources, regions, myBackgroundSources)

end subroutine getBackgroundRegions
!***********************************************
!
!********************************************** 

subroutine computebackground(value, sigma)
    real(single), intent(out) :: value, sigma

   value = backgroundMode(myImage, size(myImage, 1), &
      size(myImage, 2), 2.0, sigma) 
end subroutine computebackground


!********************************************
! GetApertureCounts
!*******************************************
subroutine GetTheCounts
   
    integer(int32) :: i      

    if (.not. associated(myBackgroundSources)) then
        return
    end if
    do i = 1, size(myBackgroundSources)
         myBackgroundSources(i)%exposure = 1.0
        call GetCounts(myImage, myBackgroundSources(i))
    end do
end subroutine GetTheCounts    


!********************************************
! GetApertureCounts
!*******************************************
subroutine GetSourceCounts(x, y, radius, counts)
    real(single), intent(in) :: x, y, radius
    real(single), intent(out) :: counts
        

   
    call GetCounts(myImage, x, y, radius, counts)
    
end subroutine GetSourceCounts    

!********************************************
! GetImageCounts
!*******************************************
real(single) Function GetImageCounts(xPos, yPos, radius)
    real(single), intent(in) :: xPos, yPos, radius
    if (.not. associated(myImage)) call fatal("myImage is not associted", "GetImageCounts")

  

    call GetCounts(myImage, xPos, yPos, radius, GetImageCounts)

end Function GetImageCounts  

!********************************************
! GetBackgroundCounts
!*******************************************
subroutine GetTheBackgroundCounts(x, y, radius, r1, r2in, counts)
    real(single), intent(in) :: x, y, radius, r1, r2in
    real(single), intent(out) :: counts
    real(single) :: r2

    r2 = r2In
   
    call GetBackgroundCounts(myImage, x, y, radius, r1, r2, counts)
    
end subroutine GetTheBackgroundCounts    

!********************************************
! deleteBackgroundList
!*******************************************
subroutine deleteBackgroundList()   
    call omsource_deleteList(myBackgroundSources)
    
end subroutine deleteBackgroundList




real(single) Function GetPSFFraction(xPos, yPos, radius)
    real(single), intent(in) :: xPos, yPos, radius

    GetPSFFraction = PSFFractionOfTotalCounts(&
              xPos, yPos, radius, 0.5)

end Function GetPSFFraction  

!*******************************************************
!
!******************************************************
real(single) Function GetTheBackgroundMode()

    GetTheBackgroundMode = omsource_GetBackgroundMode()

end Function GetTheBackgroundMode
!********************************************
! DeleteBackgroundRegions
!*******************************************
subroutine DeleteBackgroundRegions
    

    call delete(myBackgroundRegions)
  

end subroutine DeleteBackgroundRegions
subroutine InitialisePhotometry

    call InitialiseArrays

end subroutine InitialisePhotometry
subroutine Test1(n, data)
    integer, intent(in) :: n
    real data(:)
    integer i

write(*, *) "Test1"

    do i = 1, n
        write(*, *) i, data(i)
    end do
    
end subroutine Test1


subroutine ComputeSourceParameters(sources)
   type(SourceParameters), pointer :: sources(:)
 

   call omsource_computeparameters(sources)
end subroutine ComputeSourceParameters

subroutine ComputePhotometry(sources)
   type(SourceParameters), pointer :: sources(:)
 
   
   call omsource_photometry(myImage, size(sources), sources) 
 
end subroutine ComputePhotometry





!*****************************************************
!
!****************************************************

subroutine AppendToSourceList(newSources)
   type(SourceParameters), pointer :: newSources(:)
   type(SourceParameters), pointer :: tempSources(:)
   integer(int32) :: k

   if (.not. associated(newSources)) return; 
   if (.not. associated(mySources)) then
        mySources => newSources
        nullify(newSources)
        return
   end if

   
   k = size(mySources)
 
   call new(tempSources, low = 1, high = k)
 
   tempSources = mySources
   deallocate(mySources)
   nullify(mySources)
    
  
   
   
   call new(mySources, low = 1, high = k + size(newSources))
 
   mySources(1 : k) = tempSources
   deallocate(tempSources)
   nullify(tempSources)
   mySources(k + 1 : k + size(newSources)) = newSources

    


 
end subroutine AppendToSourceList







!*******************************************
! Function ComputeEpoch
!******************************************
real(double) function  ComputeEpoch(dateObs)
   
    character(19), intent(in)  :: dateObs               
    real(double)  :: JulianDate0, JulianDate
    character(19)  :: missionReferenceTime = "1998-01-01T00:00:00"
    
    ! Compute the Julian date of the mission reference time.
    JulianDate0 = ModifiedJulianDate(missionReferenceTime)

    ! Compute the Julian date of the observation
    JulianDate = ModifiedJulianDate(dateObs)

    
    ComputeEpoch = 1998.0_double + (JulianDate - JulianDate0) / 365.25_double

end function ComputeEpoch


!*******************************************************
! ModifiedJulianDate - Returns the modified Julian date
! from the string date representation
!*******************************************************
real(double) function ModifiedJulianDate(dateString)
    character(*), intent(in) :: dateString

    integer :: year            ! Stores the year from the date string
    integer :: month           ! Stores the month from the date string
    integer :: day             ! Stores the day from the date string  
    integer :: hour            ! Stores the hour from the date string
    integer :: minute          ! Stores the minute from the date string  
    integer :: second          ! Stores the second from the date string
    character(1) char          ! Used for date and time extraction 
    ! from string representation
    real(double) :: JulianDay    
    integer :: flag

    read(dateString, '(i4, a1, i2, a1, i2, a1, i2, a1, i2, a1, i2)') &
         year, char, month, char, day, char, hour, &
         char, minute, char, second     

    ! Get the modified Julian date for this date
    call sla_CLDJ(year, month, day, JulianDay, flag)

    ! Need to add on the fraction of a day corresponding to the time
    ModifiedJulianDate = JulianDay + (dble(hour) + &
         (dble(minute) + dble(second) / 60.0d0) / 60.0d0) / 24.0d0
end function ModifiedJulianDate
!*************************************************************
!
!*************************************************************
subroutine ComputeJulianDates(JDStart, JDEnd, JDMiddle)
    real(double), intent(out) :: JDStart, JDEnd, JDMiddle
    type(DataSetT) :: dataSetH
    character(20) :: string
    
    dataSetH = DataSet(trim(stringParameter("imageset")), read)


    string = trim(stringAttribute(dataSetH, "DATE-OBS"))

    JDStart = ModifiedJulianDate(string)

    string = trim(stringAttribute(dataSetH, "DATE-END"))

    JDEnd = ModifiedJulianDate(string)

 

    call release(dataSetH)


    JDMiddle = (JDStart + JDEnd) / 2.0_double

end subroutine ComputeJulianDates

end module omsource_glue














