! File:      omsourceparameter_mod
! Author:    Jorgo Bakker (jbakker@rssd.esa.int)
! Generated: Fri Aug  9 16:47:44 MEST 2002
! Usage:     C++/F95
! Example:
!   type(OmSourceParameter)          :: mySource
!   type(OmSourceParameter), pointer :: mySources(:)
!
!   call construct(mySource)         ! init contents
!   call new(mySources,low=0,high=5) ! allocate(s(0:5) and init...
!   :
!   call delete(mySources)
!   call destruct(mySource)
!use omdetectdetection_mod
module omsourceparameter_mod
   use types
   use cal
   use errorHandling
   use omsourceparam_mod

   use omsourceutils_mod
   implicit none

public

 character(*), parameter :: myModule="omsourceparameter_mod"



    type PhotometryParameters
        real(single)    :: apertureRadius              ! cal aperture radius (binned pixels)
  
        real(single)    :: calApertureRadius              ! cal aperture radius (binned pixels)
        integer         :: binX
    end type PhotometryParameters 
  

    !*****************************************************
    ! The observation details
    !*****************************************************
    type ObsDetails
        character(40) :: telescope
        character(40) :: observer
        character(40) :: object
        character(40) :: instrument
        character(40) :: obsID
        character(40) :: expID
        character(40) :: startObs
        character(40) :: endObs
        character(40) :: filter
        real(single)  :: exposure
    end type ObsDetails


    !*****************************************************
    ! The observation details
    !*****************************************************
    type ImageWindow
        integer(int32) :: x0
        integer(int32) :: y0
        real(double)   :: RA
        real(double)   :: Dec  
        real(double)   :: frameTime
        real(double)   :: deadFraction
        integer(int32) :: width
        integer(int32) :: height
        integer(int32) :: bx
        integer(int32) :: by
        logical        :: binBpe
        character(40)  :: filter
        character(40)  :: datamode
        real(double)   :: plateScale
    end type ImageWindow



  type OmCelestial
!      private
      integer(int32) :: x0,y0   ! offset in image pixels
      integer(int32) :: bx,by   ! binning in image pixels
      real(double)   :: plateScale       
      real(double)   :: rollAngle       
      real(double)   :: ra,dec  ! of centre point

      ! info as above, but shortcuts, that are heavily used
      real(double)   :: d_x0,d_y0
      real(double)   :: d_bx,d_by
      real(single)   :: sinRoll,cosRoll ! of rollangle
   end type OmCelestial




    type LsqData
        integer(int8) :: noStars
        logical(boolean) :: toFit(10)         
       
        real(single) :: backGround  
        real(single) :: sigmaX
        real(single) :: sigmaY
        real(single) :: xPos(10)
        real(single) :: yPos(10)
        real(single) :: i0(10)
        integer :: exitValue
    end type LsqData








   ! local storage types:
   type(ImageWindow), public      :: myImageWindow
   type(ObsDetails), public       :: myObsDetails
   type(OmCelestial), public      :: myCelestial
!   type(PhotometryParameters), public :: myPhotometryParameters

 
   real(single),parameter, public :: myNullValue=99999.

   ! image related stuff:

   real(single), pointer, public  :: myImage(:, :)   ! spatial
   logical(bool),pointer, public  :: myQuality(:, :) ! quality array
  


   ! The aperture radii etc...
   real(single), public           :: myApertureRadius  ! 12 unbinned pixels
   real(single), public           :: myCalRadius       
   
   real(single), public           :: myBgrRadiusIn
   real(single), public           :: myBgrRadiusOut
   real(single), public           :: myBgrMode
   real(single), public           :: myExposure
   logical, public ::  myScaleCounts = .false.
   real(single), public :: myScaleCountsRatio = 1.0 
  


   ! Some CCD parameters for computing magnitudes:
   real(double), public           :: myDeadFraction
   real(double), public           :: myFrameTime
   real(single), public           :: myDetectionRate
   real(single), public           :: myDetectionMagnitude


   ! datasets in memory:
   type(DataSetT), public         :: myImageSet

   ! determine whether we loaded sources or not?
   ! TODO: this is used for myDetectionRate/Magnitude, but
   ! if the user starts adding sources, is this still valid?
   logical, public                :: mySourcesAreLoaded
   





!   private  ! all, but (see for more information below)

   public  :: SourceParameters
   public  :: PhotometryParameters
   public :: ObsDetails   ! NB
   public :: ImageWindow
   public :: LsqData
   public  :: construct
   public  :: destruct
   public  :: new
   public  :: delete

  




!   interface construct
!      module procedure construct_od
!   end interface

!   interface destruct
!      module procedure destruct_od
!   end interface
  

   interface construct
      module procedure construct_
   end interface

   interface destruct
      module procedure destruct_
   end interface

   interface new
      module procedure new_
   end interface

   interface delete
      module procedure delete_
   end interface






 
contains

   subroutine construct_(s)
      type(SourceParameters), intent(out) :: s
      type(SourceParameters) :: s1
      s = s1
      

   end subroutine construct_



   subroutine destruct_(s)
      type(SourceParameters),intent(inout) :: s
      ! we don't have to do something in here 
      ! but keep the compiler happy
      s%xPos=0
   end subroutine destruct_



   subroutine new_(sources, low, high)
      type(SourceParameters),pointer    :: sources(:)
      integer,                intent(in) :: low,high

      integer                         :: i, status
      allocate(sources(low:high),stat=status)
!      call assertOn(status/=0,myModule//".new.sources")


      do i = low, high
         call construct(sources(i))
      end do
   end subroutine new_



   subroutine delete_(sources)
      type(SourceParameters),pointer :: sources(:)

      if (associated(sources)) then


         deallocate(sources)
         nullify(sources)
      end if
   end subroutine delete_



end module omsourceparameter_mod
