! CB TEST PROGRAM
!*********************************************************************************************************** 
module omsourcestardecon_mod

    use dal
    use cal
    use param  
    use sasdal
    use omconstants_mod
    use omallocation_mod 
    use errorHandling
    use omsmoothimage_mod
    use nonlinls_mod 
    
    use omsourceparameter_mod
    use splinefit_mod
    use ompsf_mod
    implicit none 
 
  private
  public  stardecon_deconvolve


     
type ImageData
    integer(int8) noStars
    real, pointer ::  x(:)=>Null()
    real, pointer ::  y(:)=>Null() 
    real, pointer ::  f(:)=>Null()
    real, pointer ::  weight(:)=>Null()
end type ImageData


type (ImageData), save :: lsData     

type(TwoDSpline), save :: psfSpline
 



real(single), pointer :: rCosTheta(:, :) => null(), rSinTheta(:, :) => null()
integer(int32) :: nSize, nx1, ny1

real(single) :: constant, xLower, yLower, xUpper, yUpper


contains 


 
  
!**************************************************************
! The main program
!**************************************************************
subroutine stardecon_deconvolve(binX, binY, parameters, source, image)
    integer(int32), intent(in) :: binX, binY
    type(lsqData), intent(inout) :: parameters
    type(SourceParameters), intent(inout) :: source
    real, pointer :: image(:, :)
  
    real(double) :: x(20), bl(20), bu(20)
    
    integer(int32) :: i, n
    integer(int32) :: i0, j0, il, iu, jl, ju, nx, ny
   
    integer(int32) :: width, status
 

    real(single) :: mode, sigma
 
    call CreatePSF(1023.0, 1023.0, 1.0, binX, binY,  1, 1, 100.0, psfSpline)




!    call ComputePSFWidths(psfSemiA, psfSemiB)     

 

    width = 12 / binX 

   
    nx = size(image, 1)
    ny = size(image, 2)
    mode = BackgroundMode(image, nx, ny, 2.0, sigma)

    i0 = max(0, int(source%xPos - 0.499))
    j0 = max(0, int(source%yPos - 0.499))
 
  
    il = max(0, i0 - width)
    iu = min(nx - 1, i0 + width)
    jl = max(0, j0 - 20)
    ju = min(ny - 1, j0 + width)

    
    bl(1) = max(0.0, mode - 5)
    bu(1) = mode + 5.0
    x(1) = mode

    x(2) = source%xPos
    bl(2) = x(2) - 5.0
    bu(2) = x(2) + 5.0

    x(3) = source%yPos
    bl(3) = x(3) - 5.0
    bu(3) = x(3) + 5.0

    x(4) = image(i0, j0) - mode
    bl(4) =  mode
    bu(4) =  image(i0, j0)
    
  

    x(5) = 2.0
    bl(5) = 1.0
    bu(5) = 3.0    
     
    n = 5
    write(*, *) "Gaussian function"
 
    call Deconvolve(1, image, il, iu, jl, ju, n, x, bl, bu, parameters%toFit, function2, status)
   

!return
  
    
    do i  = 1, n
    write(*, *) i, " ",  parameters%toFit(i)
    end do


    n = 6


  
  
    x(5) = 5.0
    bl(5) = 1.0
    bu(5) = 10.0    
  

    x(6) = 1.0
    bl(6) = 0.1
    bu(6) = 5.0
 
   write(*, *) "Modified Lorentzian function"
 
   call Deconvolve(1, image, il, iu, jl, ju, n, x, bl, bu, parameters%toFit, function3, status)
   

!return

   
    do i  = 1, n
    write(*, *) i, " ",  parameters%toFit(i)
    end do

 n = 6


    x(5) = 5.0
    bl(5) = 1.0
    bu(5) = 10.0    
  
   
  

    x(6) = 1.0
    bl(6) = 0.1
    bu(6) = 5.0
  write(*, *) "Moffat function"
    
  call Deconvolve(1, image, il, iu, jl, ju, n, x, bl, bu, parameters%toFit, function4, status)
   

!return
   
    do i  = 1, n
    write(*, *) i, " ",  parameters%toFit(i)
    end do

 

!x(1) = 0.98 * x(1)
!x(2) = 1.02 * x(2)
!x(3) = 0.99 * x(3)
!x(4) = 1.02 * x(4)


    call Deconvolve(1, image, il, iu, jl, ju, n, x, bl, bu, parameters%toFit, function1, status)
    parameters%exitValue = status    
    call DestructSpline(psfSpline)

 
end subroutine stardecon_deconvolve
!**************************************************
! Deconvolve
!
!
!**************************************************
subroutine Deconvolve(noStars, image, &
           il, iu, jl, ju, n, x, bl, bu, toFit, func, status)
    integer(int32), intent(in) :: noStars
    real(single), pointer :: image(:, :)
    integer(int32), intent(in) :: il, iu, jl, ju
    integer(int32), intent(in) :: n
    real(double), intent(inout) :: x(1 : n)
    real(double), intent(inout) :: bl(1 : n), bu(1 : n)
    logical(boolean), intent(in) :: toFit(1 : n)
    integer(int32), intent(inout) :: status
    integer(int32) :: i, j, m
    real(double), pointer :: f(:)=>Null()
    external func
    m = (iu - il + 1) * (ju - jl + 1)
    allocate(lsData%x(1 : m)) 
    allocate(lsData%y(1 : m)) 
    allocate(lsData%f(1 : m)) 
    allocate(lsData%weight(1 : m)) 

    m = 0
    lsData%noStars = noStars
  
    do i = il, iu
        do j = jl , ju
            m = m + 1

            lsData%x(m) = float(i) + 0.5
            lsData%y(m) = float(j) + 0.5
            lsData%f(m) = image(i, j)
            if(lsData%f(m) > 0.0) then
               lsData%weight(m) = sqrt(lsData%f(m))
            else
 
               lsData%weight(m) = 1.0
            end if
!           lsData%weight(m) = 1.0
   
         end do
    end do

    allocate(f(1 : m))
!    do i = 1, 3   


    write(*, *) "Input values:"
    do i = 1, n    
        write(*, *) "x(", i, ")=", x(i)
    end do
  




  call nonlin(x, n, m, func, bl, bu, toFit, f, status)
!  call nonlin(x, n, m, Function1, bl, bu, status, f)

!    end do
    
    write(*, *) "Solution:"
    do i = 1, n    
        write(*, *) "x(", i, ")=", x(i)
    end do
    write(*, *) "Sum of squares of residuals=", DOT_PRODUCT(f, f)
    write(*, *) "status=", status   
    deallocate(f, lsData%x, lsData%y, lsdata%f)
    nullify(f)
    nullify(lsData%x)
    nullify(lsData%y)
    nullify(lsData%f)
    nullify(f)

 
end subroutine Deconvolve

!***********************************************************
!
!
!
!**********************************************************
subroutine Function1(x, n, f, m, ctrl, c)
    real(double), intent(in) :: x(:)
    integer, intent(in) :: n
    real(double), intent(out) :: f(:)
    integer, intent(in) :: m
    integer, intent(inout) :: ctrl
    real(double), intent(out) :: c(:, :)
    integer i, j, k
    real(double) :: fit

    if (ctrl .eq. 2) then
        ctrl = 0
        return
    end if
    
    do i = 1, m         
        fit = x(4) * InterpolateSpline((lsData%x(i) - real(x(2))), &
                      (lsData%y(i) - real(x(3))), psfSpline)    
              ! The first star     
          
        do j = 2, lsData%noStars
            k = 3 * j - 1 
            fit = fit + &   ! The remaining ones
                 x(k + 2) * InterpolateSpline((lsData%x(i) - real(x(k))), &
                      (lsData%y(i) - real(x(k + 1))), psfSpline) 
        end do
        f(i) = (lsData%f(i) - (x(1) + fit)) * lsData%weight(i)
     
    end do  
          
end subroutine Function1      


!***********************************************************
!
!
!
!**********************************************************
subroutine Function2(x, n, f, m, ctrl, c)
    real(double), intent(in) :: x(:)
    integer, intent(in) :: n
    real(double), intent(out) :: f(:)
    integer, intent(in) :: m
    integer, intent(inout) :: ctrl
    real(double), intent(out) :: c(:, :)
    integer i
    real(double) :: fit

    if (ctrl .eq. 2) then
        ctrl = 0
        return
    end if
    

    do i = 1, m         
        fit = x(1) + Gaussian(lsData%x(i), lsData%y(i), x(2), x(3), x(4), x(5)) 
        f(i) = (lsData%f(i) - fit) * lsData%weight(i)
     
    end do  
          
end subroutine Function2      


!***********************************************************
!
! Modified Lorentzian
!
!**********************************************************
subroutine Function3(x, n, f, m, ctrl, c)
    real(double), intent(in) :: x(:)
    integer, intent(in) :: n
    real(double), intent(out) :: f(:)
    integer, intent(in) :: m
    integer, intent(inout) :: ctrl
    real(double), intent(out) :: c(:, :)
    integer i
    real(double) :: fit

    if (ctrl .eq. 2) then
        ctrl = 0
        return
    end if
    

    do i = 1, m         
        fit = x(1) + Lorentzian(lsData%x(i), lsData%y(i), x(2), x(3), x(4), x(5), x(6)) 
        f(i) = (lsData%f(i) - fit) * lsData%weight(i)
     
    end do  
          
end subroutine Function3      


!***********************************************************
!
! Moffat
!
!**********************************************************
subroutine Function4(x, n, f, m, ctrl, c)
    real(double), intent(in) :: x(:)
    integer, intent(in) :: n
    real(double), intent(out) :: f(:)
    integer, intent(in) :: m
    integer, intent(inout) :: ctrl
    real(double), intent(out) :: c(:, :)
    integer i
    real(double) :: fit

    if (ctrl .eq. 2) then
        ctrl = 0
        return
    end if
    

    do i = 1, m         
        fit = x(1) + Moffat(lsData%x(i), lsData%y(i), x(2), x(3), x(4), x(5), x(6)) 
        f(i) = (lsData%f(i) - fit) * lsData%weight(i)
     
    end do  
          
end subroutine Function4      





!*********************************************
! Evaluates a * exp(-(x - x0) ** 2 + (y - y0) ** 2) /sigma)
! xc(1) = x0
! xc(2) = y0
! xc(3) = a
! xc(4) = sigma
!*******************************************************
real(kind=double) Function Gaussian(x, y, xc, yc, a, b)
    real(single), intent(in) :: x, y
    real(double), intent(in) :: xc, yc, a, b
    real(double) :: arg

    arg = ((x - xc) ** 2 + (y - yc) ** 2) / b 

    if (arg < 30.0) then
        Gaussian = a * exp(-arg)
    else
        Gaussian = 0.0_double
    end if

end Function Gaussian
!*********************************************
! Evaluates a * exp(-(x - x0) ** 2 + (y - y0) ** 2) /sigma)
! xc(1) = x0
! xc(2) = y0
! xc(3) = a
! xc(4) = sigma
!*******************************************************
real(kind=double) Function Lorentzian(x, y, xc, yc, a, b, c)
    real(single), intent(in) :: x, y
    real(double), intent(in) :: xc, yc, a, b, c
    real(double) :: arg

    arg = ((x - xc) ** 2 + (y - yc) ** 2) / (b * b) 
    

    Lorentzian = a / (1.0_double + arg ** c)

end Function Lorentzian

!*********************************************
! Evaluates a * exp(-(x - x0) ** 2 + (y - y0) ** 2) /sigma)
! xc(1) = x0
! xc(2) = y0
! xc(3) = a
! xc(4) = sigma
!*******************************************************
real(kind=double) Function Moffat(x, y, xc, yc, a, b, c)
    real(single), intent(in) :: x, y
    real(double), intent(in) :: xc, yc, a, b, c
    real(double) :: arg

    arg = ((x - xc) ** 2 + (y - yc) ** 2) / (b * b) 
    

    Moffat = a / (1.0_double + arg) ** c

end Function Moffat

!********************************************************
! Subroutine DetectPointSources
! Sets the pixels on the levelImage to the source number
! (0=background +for point-sources, - for extended sources)
!********************************************************
subroutine DetectPointSources(nx, ny, image)

    integer(kind=int32), intent(in) :: nx, ny
    real(kind=single), intent(in) :: image(0 : nx - 1, 0 : ny - 1)
    
   
    real(kind=single), parameter :: nSigma = 3.0
    
           
    real(kind=single) :: mode
 
    integer(kind=int16) :: sourceNumber
    real(single) :: minImage, maxImage
    logical, pointer :: maskImage(:, :) => null()
    
    real(kind=single), pointer :: tempImage(: , :) => Null()
 
 

    call Initialise(nx, ny) 
    
     
    allocate(maskImage(0 : nx1, 0 : ny1))
    
    
    write(errstr,'(a)') "Looking for point sources"
    call message(AppMsg, Verbose)
           
    sourceNumber = 0    

    ! Look for brightest sources first
                 
    tempImage => Allocate2DSingle(0, nx - 1, 0, ny - 1, "tempImage") 
    tempImage = image
    call SmoothImage(tempImage, nx, ny, 2 / binX, 2 / binY)  ! was 2, 2

    mode = minval(image) + 5.0
  
    miniMage = max(2.0, mode + nSigma * sqrt(mode))    
   
    
   
           

    maxImage = maxval(tempImage)    
  
    maskImage = .false.
    
    constant = 1.0  
    call SearchForPointSources(tempImage, &
                              maskImage, &
                              minImage, maxImage)



    deallocate(tempImage)
    deallocate(maskImage)
    nullify(maskImage)
    deallocate(rCosTheta, rSinTheta)
    nullify(rCosTheta)
    nullify(rSinTheta)
   
    ! The number of detected "point" sources
  

end subroutine DetectPointSources
!***********************************************************
!
!***********************************************************
subroutine SearchForPointSources(image, &
                                maskImage, &
                                minImage, maxImage)
    
    real, pointer :: image(:, :)
   
    
    logical, pointer :: maskImage(:, :)
    real(single), intent(in) :: minImage
    real(single), intent(inout) :: maxImage
  
         
    integer(kind=int32) :: i0, j0
    integer(kind=int8) :: dx
    real(single) :: delta 
    integer(int8) noPeeks
    integer(int16) :: i1, j1, jl, ju
  
 
    integer(kind=int32) :: maxIndex(1 : 2)   
 
  
    
    dx = 2 / binX
    
    delta = 1.50  ! Maximum allowed ratio of major to minor axis for a point source 
   


  
    where(image < minImage) maskImage = .true.    
    
    where(image > maxImage) maskImage = .true.    

    noPeeks = 0
    do
        ! Detect sources in order of decreasing maximum pixel value    
        ! Get the indices of the maximum pixel- excluding
        ! those pixels where maskImage = true
        maxIndex = maxloc(image, mask = .not. maskImage)
         
        i0 = maxIndex(1) - 1
        j0 = maxIndex(2) - 1

        if (i0 .lt. 0 .or. j0 .lt. 0) exit

        maxImage = image(i0, j0)    
  
        if (maxImage .le. minImage) exit 
            
        maskImage(i0, j0) = .true. 
        
                                  
        ! See if this pixel is centred on a source
        if (IsPointSource(i0, j0, image)) then
            noPeeks = noPeeks + 1            
            write(*, *) "Peek found at ", i0, ", ", j0



   
                jl = max(0, j0 - dx)  ! was dx4
                ju = min(ny1, j0 + dx) ! was dx4
    
   
                forall (i1 = max(0, i0 - dx) : min(nx1, i0 + dx)) ! was dx4 
                    forall (j1 = jl : ju)
  
 
                            maskImage(i1, j1) = .true.                                  
   
                    end forall
                end forall



            
        end if
          
    end do    

write(*, *) "No peeks found=", noPeeks
    
  
   
end subroutine SearchForPointSources

!***********************************************************
! Function IsPointSource
! Determines if the underlying pixels on the image
! constitute a point-source
! return TRUE is so, FALSE if not
!***********************************************************
logical Function IsPointSource(i0, j0, image)
  
    integer(kind=int32), intent(inout) :: i0, j0
    real(single), intent(in)  :: image(0 : nx1, 0 : ny1)
    !************************************
    ! Local variables
    !************************************  
    real(double) :: f(nSize), mean(3), sigma(3)
    real(single) :: xCentre, yCentre, xPos, yPos
    integer(kind=int32) :: i
    integer(kind=int32) :: j, k
!******************************************************
! for test
!*******************************************
!    real(double), save  :: minF = 0.0_double, maxF = 0.0_double, fratio = 0.0_double
!    integer, save :: count = 0
!write(*, *) "IsPointsource line 233, nx1=", nx1, " ny1=", ny1


    xCentre = real(i0) + 0.5
    yCentre = real(j0) + 0.5

    !****************************************************
    ! 
    !****************************************************
    do i = 2, 1, -1

        f = 0.0
        k = 0  
        do j = 1, nSize
           
            xPos = xCentre + rCosTheta(j, i)
            if (xPos .lt. xLower .or. xPos .gt. xUpper) cycle 
            yPos = yCentre + rSinTheta(j, i)
            if (yPos .lt. yLower .or. yPos .gt. yUpper) cycle 
            k = k + 1 
            
            f(k) = Interpolate(xPos, yPos, image)
            
    
        end do
        if (k .lt. 6) then
            IsPointSource = .false.
            return
        end if   

        call computeStatistics(k, f, mean(i), sigma(i))
!        if (mean(i) / sigma(i) < 7.0) then
!            IsPointSource = .false.
!            return
!        end if
!        write(*, *) "minval f=", minval(f), " max=", maxval(f), " ratio=", (minval(f) / maxval(f))

!write(*, *) i, " mean=", mean(i), " sigma=", sigma(i)       
   
   
        if (i == 2) cycle
!        write(*, *) "radius(", i, ")=", (real(i * 2) / real(binx))
        if (mean(i + 1) >= 0.9 * mean(i)) then
 
            IsPointSource = .false.

!write(*, *) "rejecting point-mean(", (i+1), ")=", mean(i+1), " mean(", i, ")=", mean(i)

            return
        end if

        if ((mean(i) - mean(i + 1)) <= &
             constant * sqrt(sigma(i) + sigma(i + 1))) then  ! was 0.5
              
 
!write(*, *) "rejecting point-mean(", (i+1), ")=", mean(i+1),  " mean(", i, ")=", mean(i)
!write(*, *) "sigma(", (i+1), ")=", sigma(i+1),  " sigma(", i, ")=", sigma(i)
!write(*, *) " sqrt(sigma(i) + sigma(i + 1)))=",  sqrt(sigma(i) + sigma(i + 1)) 
                       
            IsPointSource = .false.

            return
        end if
    end do

!if (k == nSize) then
!    minF= minF + minval(f)
!    maxF = maxF + maxval(f)
!    fratio = fratio + minval(f) / maxval(f)
!    count = count + 1
!    write(*, *) "mean min f=", minf / dble(count), " max meanf=", maxf / dble(count), &
!     " mean ratio=", fratio / dble(count)
!end if
    IsPointSource = .true.
    
end Function IsPointSource





!**************************************************
!
!**************************************************
subroutine ComputeStatistics(n, x, mean, sigma)
    integer, intent(in) :: n
    real(double), intent(in) :: x(1 : n)
    real(double), intent(out) :: mean, sigma

    mean = sum(x) / dble(n)
    
    sigma = (sum((x - mean) ** 2)) / dble(n - 1)
end subroutine ComputeStatistics
  
!*************************************************
!
!*************************************************
real(single) Function Interpolate(xPos, yPos, image)
    real(single), intent(in) :: xPos, yPos
    real(single), intent(in) :: image(0 : nx1, 0 : ny1)
    integer(int32) ::  i0, j0, i1, j1
    real(double) :: t, u, oneMinusT, oneMinusU

    i0 = int(xPos)
    i1 = i0 + 1    
    j0 = int(yPos)
    j1 = j0 + 1
    t = xPos - dble(i0)  
    oneMinusT = 1.0_double - t
    u = yPos - dble(j0)
    oneMinusU = 1.0_double - u
  
    Interpolate =  oneMinusT * &
                   oneMinusU * &
                   image(i0, j0) + t *   &
                   oneMinusU * image(i1, j0) + t * u * &
                                   image(i1, j1) + &
                   oneMinusT  * u * &              
                   image(i0, j1)

end Function Interpolate


!******************************************
!
!*****************************************
subroutine Initialise(nx, ny)
    integer(int32), intent(in) :: nx, ny


    integer(int8) :: i, j
    real(single) :: radius, angle, deltaAngle

    nx1 = nx - 1
    ny1 = ny - 1
 
    xLower = 0.1
    yLower = 0.1
    xUpper = real(nx) - 2.1
    yUpper = real(ny) - 2.1

    nSize = 12
    rCosTheta => Allocate2DSingle(1, nSize, 1, 3, "rCosTheta")
    rSinTheta => Allocate2DSingle(1, nSize, 1, 3, "rCosTheta")

    deltaAngle = 360.0 / real(nSize)
    
    do i = 1, 3
        radius = real(2 * i) / real(binX)
    
        do j = 1, nSize
            angle = real(j - 1) * deltaAngle * degreesToRadians 
            rCosTheta(j, i) = radius * cos(angle)
            rSinTheta(j, i) = radius * sin(angle)
        end do
    end do

end subroutine Initialise



end module omsourcestardecon_mod




   





